@@ -46,9 +46,8 @@ typedef struct {
 extern PyTypeObject* igraphmodule_ARPACKOptionsType;
 extern PyObject* igraphmodule_arpack_options_default;
 
-int igraphmodule_ARPACKOptions_register_type();
+int igraphmodule_ARPACKOptions_register_type(void);
 
-PyObject* igraphmodule_ARPACKOptions_new(void);
 igraph_arpack_options_t *igraphmodule_ARPACKOptions_get(igraphmodule_ARPACKOptionsObject *self);
 
 #endif
