@@ -37,28 +37,16 @@
  * \defgroup python_interface_edgeseq Edge sequence object
  */
 
-PyTypeObject igraphmodule_EdgeSeqType;
+PyTypeObject* igraphmodule_EdgeSeqType;
+
+PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject *args);
 
 /**
  * \ingroup python_interface_edgeseq
- * \brief Allocate a new edge sequence object for a given graph
- * \param g the graph object being referenced
- * \return the allocated PyObject
+ * \brief Checks whether the given Python object is an edge sequence
  */
-PyObject* igraphmodule_EdgeSeq_new(PyTypeObject *subtype,
-	PyObject *args, PyObject *kwds) {
-  igraphmodule_EdgeSeqObject* o;
-
-  o=(igraphmodule_EdgeSeqObject*)PyType_GenericNew(subtype, args, kwds);
-  if (o == NULL) return NULL;
-
-  igraph_es_all(&o->es, IGRAPH_EDGEORDER_ID);
-  o->gref=0;
-  o->weakreflist=0;
-
-  RC_ALLOC("EdgeSeq", o);
-
-  return (PyObject*)o;
+int igraphmodule_EdgeSeq_Check(PyObject* obj) {
+  return obj ? PyObject_IsInstance(obj, (PyObject*)igraphmodule_EdgeSeqType) : 0;
 }
 
 /**
@@ -70,76 +58,87 @@ igraphmodule_EdgeSeqObject*
 igraphmodule_EdgeSeq_copy(igraphmodule_EdgeSeqObject* o) {
   igraphmodule_EdgeSeqObject *copy;
 
-  copy=(igraphmodule_EdgeSeqObject*)PyType_GenericNew(Py_TYPE(o), 0, 0);
-  if (copy == NULL) return NULL;
+  copy = (igraphmodule_EdgeSeqObject*) PyType_GenericNew(Py_TYPE(o), 0, 0);
+  if (copy == NULL) {
+    return NULL;
+  }
 
   if (igraph_es_type(&o->es) == IGRAPH_ES_VECTOR) {
-    igraph_vector_t v;
-    if (igraph_vector_copy(&v, o->es.data.vecptr)) {
+    igraph_vector_int_t v;
+    if (igraph_vector_int_init_copy(&v, o->es.data.vecptr)) {
       igraphmodule_handle_igraph_error();
       return 0;
     }
     if (igraph_es_vector_copy(&copy->es, &v)) {
       igraphmodule_handle_igraph_error();
-      igraph_vector_destroy(&v);
+      igraph_vector_int_destroy(&v);
       return 0;
     }
-    igraph_vector_destroy(&v);
+    igraph_vector_int_destroy(&v);
   } else {
     copy->es = o->es;
   }
 
   copy->gref = o->gref;
-  if (o->gref) Py_INCREF(o->gref);
+  if (o->gref) {
+    Py_INCREF(o->gref);
+  }
+
   RC_ALLOC("EdgeSeq(copy)", copy);
 
   return copy;
 }
 
-
 /**
  * \ingroup python_interface_edgeseq
  * \brief Initialize a new edge sequence object for a given graph
  * \return the initialized PyObject
  */
-int igraphmodule_EdgeSeq_init(igraphmodule_EdgeSeqObject *self,
-  PyObject *args, PyObject *kwds) {
+int igraphmodule_EdgeSeq_init(igraphmodule_EdgeSeqObject *self, PyObject *args, PyObject *kwds) {
   static char *kwlist[] = { "graph", "edges", NULL };
-  PyObject *g, *esobj=Py_None;
+  PyObject *g, *esobj = Py_None;
   igraph_es_t es;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O", kwlist,
-    &igraphmodule_GraphType, &g, &esobj))
-      return -1;
+    igraphmodule_GraphType, &g, &esobj)) {
+    return -1;
+  }
 
   if (esobj == Py_None) {
     /* If es is None, we are selecting all the edges */
     igraph_es_all(&es, IGRAPH_EDGEORDER_ID);
   } else if (PyLong_Check(esobj)) {
     /* We selected a single edge */
-    long int idx = PyLong_AsLong(esobj);
+    igraph_integer_t idx;
+    
+    if (igraphmodule_PyObject_to_integer_t(esobj, &idx)) {
+      return -1;
+    }
+
     if (idx < 0 || idx >= igraph_ecount(&((igraphmodule_GraphObject*)g)->g)) {
       PyErr_SetString(PyExc_ValueError, "edge index out of range");
       return -1;
     }
-    igraph_es_1(&es, (igraph_integer_t)idx);
+
+    igraph_es_1(&es, idx);
   } else {
-	/* We selected multiple edges */
-    igraph_vector_t v;
+   /* We selected multiple edges */
+    igraph_vector_int_t v;
     igraph_integer_t n = igraph_ecount(&((igraphmodule_GraphObject*)g)->g);
-    if (igraphmodule_PyObject_to_vector_t(esobj, &v, 1))
+    if (igraphmodule_PyObject_to_vector_int_t(esobj, &v)) {
       return -1;
-    if (!igraph_vector_isininterval(&v, 0, n-1)) {
-      igraph_vector_destroy(&v);
+    }
+    if (!igraph_vector_int_isininterval(&v, 0, n-1)) {
+      igraph_vector_int_destroy(&v);
       PyErr_SetString(PyExc_ValueError, "edge index out of range");
       return -1;
     }
     if (igraph_es_vector_copy(&es, &v)) {
       igraphmodule_handle_igraph_error();
-      igraph_vector_destroy(&v);
+      igraph_vector_int_destroy(&v);
       return -1;
     }
-    igraph_vector_destroy(&v);
+    igraph_vector_int_destroy(&v);
   }
 
   self->es = es;
@@ -154,32 +153,36 @@ int igraphmodule_EdgeSeq_init(igraphmodule_EdgeSeqObject *self,
  * \brief Deallocates a Python representation of a given edge sequence object
  */
 void igraphmodule_EdgeSeq_dealloc(igraphmodule_EdgeSeqObject* self) {
-  if (self->weakreflist != NULL)
+  RC_DEALLOC("EdgeSeq", self);
+
+  if (self->weakreflist != NULL) {
     PyObject_ClearWeakRefs((PyObject *)self);
+  }
+
   if (self->gref) {
     igraph_es_destroy(&self->es);
-    Py_DECREF(self->gref);
-    self->gref=0;
   }
-  Py_TYPE(self)->tp_free((PyObject*)self);
 
-  RC_DEALLOC("EdgeSeq", self);
+  Py_CLEAR(self->gref);
+  PY_FREE_AND_DECREF_TYPE(self, igraphmodule_EdgeSeqType);
 }
 
 /**
  * \ingroup python_interface_edgeseq
  * \brief Returns the length of the sequence (i.e. the number of edges in the graph)
  */
-int igraphmodule_EdgeSeq_sq_length(igraphmodule_EdgeSeqObject* self) {
+Py_ssize_t igraphmodule_EdgeSeq_sq_length(igraphmodule_EdgeSeqObject* self) {
   igraph_t *g;
   igraph_integer_t result;
 
-  g=&GET_GRAPH(self);
+  g = &GET_GRAPH(self);
+
   if (igraph_es_size(g, &self->es, &result)) {
     igraphmodule_handle_igraph_error();
     return -1;
   }
-  return (int)result;
+
+  return result;
 }
 
 /**
@@ -191,25 +194,29 @@ PyObject* igraphmodule_EdgeSeq_sq_item(igraphmodule_EdgeSeqObject* self,
   igraph_t *g;
   igraph_integer_t idx = -1;
 
-  if (!self->gref) return NULL;
-  g=&GET_GRAPH(self);
+  if (!self->gref) {
+    return NULL;
+  }
+
+  g = &GET_GRAPH(self);
+
   switch (igraph_es_type(&self->es)) {
     case IGRAPH_ES_ALL:
       if (i < 0) {
         i = igraph_ecount(g) + i;
       }
       if (i >= 0 && i < igraph_ecount(g)) {
-        idx = (igraph_integer_t)i;
+        idx = i;
       }
       break;
 
     case IGRAPH_ES_VECTOR:
     case IGRAPH_ES_VECTORPTR:
       if (i < 0) {
-        i = igraph_vector_size(self->es.data.vecptr) + i;
+        i = igraph_vector_int_size(self->es.data.vecptr) + i;
       }
-      if (i >= 0 && i < igraph_vector_size(self->es.data.vecptr)) {
-        idx = (igraph_integer_t)VECTOR(*self->es.data.vecptr)[i];
+      if (i >= 0 && i < igraph_vector_int_size(self->es.data.vecptr)) {
+        idx = VECTOR(*self->es.data.vecptr)[i];
       }
       break;
 
@@ -219,19 +226,28 @@ PyObject* igraphmodule_EdgeSeq_sq_item(igraphmodule_EdgeSeqObject* self,
       }
       break;
 
-    case IGRAPH_ES_SEQ:
+    case IGRAPH_ES_RANGE:
       if (i < 0) {
-        i = self->es.data.seq.to - self->es.data.seq.from + i;
+        i = self->es.data.range.end - self->es.data.range.start + i;
       }
-      if (i >= 0 && i < self->es.data.seq.to - self->es.data.seq.from) {
-        idx = self->es.data.seq.from + (igraph_integer_t)i;
+      if (i >= 0 && i < self->es.data.range.end - self->es.data.range.start) {
+        idx = self->es.data.range.start + i;
       }
       break;
 
+    case IGRAPH_ES_NONE:
+      break;
+
     /* TODO: IGRAPH_ES_PAIRS, IGRAPH_ES_ADJ, IGRAPH_ES_PATH,
        IGRAPH_ES_MULTIPATH - someday :) They are unused yet in the Python
        interface */
+
+    default:
+      return PyErr_Format(
+        igraphmodule_InternalError, "unsupported edge selector type: %d", igraph_es_type(&self->es)
+      );
   }
+
   if (idx < 0) {
     PyErr_SetString(PyExc_IndexError, "edge index out of range");
     return NULL;
@@ -253,58 +269,100 @@ PyObject* igraphmodule_EdgeSeq_attribute_names(igraphmodule_EdgeSeqObject* self)
 PyObject* igraphmodule_EdgeSeq_get_attribute_values(igraphmodule_EdgeSeqObject* self, PyObject* o) {
   igraphmodule_GraphObject *gr = self->gref;
   PyObject *result=0, *values, *item;
-  long int i, n;
+  Py_ssize_t i, n;
 
-  if (!igraphmodule_attribute_name_check(o))
+  if (!igraphmodule_attribute_name_check(o)) {
     return 0;
+  }
 
   PyErr_Clear();
-  values=PyDict_GetItem(ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_EDGE], o);
+  values = PyDict_GetItem(ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_EDGE], o);
   if (!values) {
     PyErr_SetString(PyExc_KeyError, "Attribute does not exist");
     return NULL;
-  } else if (PyErr_Occurred()) return NULL;
+  } else if (PyErr_Occurred()) {
+    return NULL;
+  }
 
   switch (igraph_es_type(&self->es)) {
     case IGRAPH_ES_NONE:
       n = 0;
       result = PyList_New(0);
+      if (!result) {
+        return 0;
+      }
       break;
 
     case IGRAPH_ES_ALL:
       n = PyList_Size(values);
       result = PyList_New(n);
-      if (!result) return 0;
+      if (!result) {
+        return 0;
+      }
 
-      for (i=0; i<n; i++) {
-          item = PyList_GET_ITEM(values, i);
-          Py_INCREF(item);
-          PyList_SET_ITEM(result, i, item);
+      for (i = 0; i < n; i++) {
+        item = PyList_GetItem(values, i);
+        if (!item) {
+          Py_DECREF(result);
+          return 0;
+        }
+
+        Py_INCREF(item);
+
+        if (PyList_SetItem(result, i, item)) {
+          Py_DECREF(item);
+          Py_DECREF(result);
+          return 0;
+        }
       }
       break;
 
     case IGRAPH_ES_VECTOR:
     case IGRAPH_ES_VECTORPTR:
-      n = igraph_vector_size(self->es.data.vecptr);
+      n = igraph_vector_int_size(self->es.data.vecptr);
       result = PyList_New(n);
-      if (!result) return 0;
+      if (!result) {
+        return 0;
+      }
+
+      for (i = 0; i < n; i++) {
+        item = PyList_GetItem(values, VECTOR(*self->es.data.vecptr)[i]);
+        if (!item) {
+          Py_DECREF(result);
+          return 0;
+        }
 
-      for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, (long)VECTOR(*self->es.data.vecptr)[i]);
         Py_INCREF(item);
-        PyList_SET_ITEM(result, i, item);
+
+        if (PyList_SetItem(result, i, item)) {
+          Py_DECREF(item);
+          Py_DECREF(result);
+          return 0;
+        }
       }
       break;
 
-    case IGRAPH_ES_SEQ:
-      n = self->es.data.seq.to - self->es.data.seq.from;
+    case IGRAPH_ES_RANGE:
+      n = self->es.data.range.end - self->es.data.range.start;
       result = PyList_New(n);
-      if (!result) return 0;
+      if (!result) {
+        return 0;
+      }
 
-      for (i=0; i<n; i++) {
-        item = PyList_GET_ITEM(values, (long)self->es.data.seq.from+i);
+      for (i = 0; i < n; i++) {
+        item = PyList_GetItem(values, self->es.data.range.start + i);
+        if (!item) {
+          Py_DECREF(result);
+          return 0;
+        }
+        
         Py_INCREF(item);
-        PyList_SET_ITEM(result, i, item);
+
+        if (PyList_SetItem(result, i, item)) {
+          Py_DECREF(item);
+          Py_DECREF(result);
+          return 0;
+        }
       }
       break;
 
@@ -316,19 +374,16 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values(igraphmodule_EdgeSeqObject*
 }
 
 PyObject* igraphmodule_EdgeSeq_is_all(igraphmodule_EdgeSeqObject* self) {
-  if (igraph_es_is_all(&self->es))
+  if (igraph_es_is_all(&self->es)) {
     Py_RETURN_TRUE;
-  Py_RETURN_FALSE;
+  } else {
+    Py_RETURN_FALSE;
+  }
 }
 
 PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeqObject *self, PyObject *o) {
   Py_ssize_t index;
-
-  /* Handle integer indices according to the sequence protocol */
-  if (PyIndex_Check(o)) {
-    index = PyNumber_AsSsize_t(o, 0);
-    return igraphmodule_EdgeSeq_sq_item(self, index);
-  }
+  PyObject *index_o;
 
   /* Handle strings according to the mapping protocol */
   if (PyBaseString_Check(o)) {
@@ -346,9 +401,26 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeq
 
     result = igraphmodule_EdgeSeq_select(self, args);
     Py_DECREF(args);
+
     return result;
   }
 
+  /* Handle integer indices according to the sequence protocol */
+  index_o = PyNumber_Index(o);
+  if (index_o) {
+    index = PyLong_AsSsize_t(index_o);
+    if (PyErr_Occurred()) {
+      Py_DECREF(index_o);
+      return NULL;
+    } else {
+      Py_DECREF(index_o);
+      return igraphmodule_EdgeSeq_sq_item(self, index);
+    }
+  } else {
+    /* clear TypeError raised by PyNumber_Index() */
+    PyErr_Clear();
+  }
+
   /* Handle everything else according to the mapping protocol */
   return igraphmodule_EdgeSeq_get_attribute_values(self, o);
 }
@@ -359,18 +431,21 @@ PyObject* igraphmodule_EdgeSeq_get_attribute_values_mapping(igraphmodule_EdgeSeq
 int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject* self, PyObject* attrname, PyObject* values) {
   PyObject *dict, *list, *item;
   igraphmodule_GraphObject *gr;
-  igraph_vector_t es;
-  long i, j, n, no_of_edges;
+  igraph_vector_int_t es;
+  Py_ssize_t i, j, n;
+  igraph_integer_t no_of_edges;
 
   gr = self->gref;
   dict = ATTR_STRUCT_DICT(&gr->g)[ATTRHASH_IDX_EDGE];
 
-  if (!igraphmodule_attribute_name_check(attrname))
+  if (!igraphmodule_attribute_name_check(attrname)) {
     return -1;
+  }
 
   if (values == 0) {
-    if (igraph_es_type(&self->es) == IGRAPH_ES_ALL)
+    if (igraph_es_type(&self->es) == IGRAPH_ES_ALL) {
       return PyDict_DelItem(dict, attrname);
+    }
     PyErr_SetString(PyExc_TypeError, "can't delete attribute from an edge sequence not representing the whole graph");
     return -1;
   }
@@ -380,19 +455,27 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
      * single element (the value itself) and then call ourselves again */
     int result;
     PyObject *newList = PyList_New(1);
-    if (newList == 0) return -1;
+    if (newList == 0) {
+      return -1;
+    }
     Py_INCREF(values);
-    PyList_SET_ITEM(newList, 0, values);    /* reference stolen here */
+
+    if (PyList_SetItem(newList, 0, values)) {  /* reference stolen here */
+      return -1;
+    }
+
     result = igraphmodule_EdgeSeq_set_attribute_values_mapping(self, attrname, newList);
     Py_DECREF(newList);
     return result;
   }
 
-  n=PySequence_Size(values);
-  if (n<0) return -1;
+  n = PySequence_Size(values);
+  if (n < 0) {
+    return -1;
+  }
 
   if (igraph_es_type(&self->es) == IGRAPH_ES_ALL) {
-    no_of_edges = (long)igraph_ecount(&gr->g);
+    no_of_edges = igraph_ecount(&gr->g);
     if (n == 0 && no_of_edges > 0) {
       PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
       return -1;
@@ -402,10 +485,14 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0, j=0; i<no_of_edges; i++, j++) {
-        if (j == n) j = 0;
+      for (i = 0, j = 0; i < no_of_edges; i++, j++) {
+        if (j == n) {
+          j = 0;
+        }
         item = PySequence_GetItem(values, j);
-        if (item == 0) return -1;
+        if (item == 0) {
+          return -1;
+        }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
         if (PyList_SetItem(list, i, item)) {
           Py_DECREF(item);
@@ -416,13 +503,23 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
       /* We don't have attributes with the given name yet. Create an entry
        * in the dict, create a new list and copy everything */
       list = PyList_New(no_of_edges);
-      if (list == 0) return -1;
-      for (i=0, j=0; i<no_of_edges; i++, j++) {
-        if (j == n) j = 0;
+      if (list == 0) {
+        return -1;
+      }
+      for (i = 0, j = 0; i < no_of_edges; i++, j++) {
+        if (j == n) {
+          j = 0;
+        }
         item = PySequence_GetItem(values, j);
-        if (item == 0) { Py_DECREF(list); return -1; }
+        if (item == 0) {
+          Py_DECREF(list); return -1;
+        }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
-        PyList_SET_ITEM(list, i, item);
+        if (PyList_SetItem(list, i, item)) {
+          Py_DECREF(item);
+          Py_DECREF(list);
+          return -1;
+        }
         /* PyList_SET_ITEM stole a reference to the item automatically */
       }
       if (PyDict_SetItem(dict, attrname, list)) {
@@ -434,60 +531,78 @@ int igraphmodule_EdgeSeq_set_attribute_values_mapping(igraphmodule_EdgeSeqObject
   } else {
     /* We are working with a subset of the graph. Convert the sequence to a
      * vector and loop through it */
-    if (igraph_vector_init(&es, 0)) {
+    if (igraph_vector_int_init(&es, 0)) {
       igraphmodule_handle_igraph_error();
       return -1;
     }
     if (igraph_es_as_vector(&gr->g, self->es, &es)) {
       igraphmodule_handle_igraph_error();
-      igraph_vector_destroy(&es);
+      igraph_vector_int_destroy(&es);
       return -1;
     }
-    no_of_edges = (long)igraph_vector_size(&es);
+    no_of_edges = igraph_vector_int_size(&es);
     if (n == 0 && no_of_edges > 0) {
       PyErr_SetString(PyExc_ValueError, "sequence must not be empty");
-      igraph_vector_destroy(&es);
+      igraph_vector_int_destroy(&es);
       return -1;
     }
     /* Check if we already have attributes with the given name */
     list = PyDict_GetItem(dict, attrname);
     if (list != 0) {
       /* Yes, we have. Modify its items to the items found in values */
-      for (i=0, j=0; i<no_of_edges; i++, j++) {
-        if (j == n) j = 0;
+      for (i = 0, j = 0; i < no_of_edges; i++, j++) {
+        if (j == n) {
+          j = 0;
+        }
         item = PySequence_GetItem(values, j);
-        if (item == 0) { igraph_vector_destroy(&es); return -1; }
+        if (item == 0) {
+          igraph_vector_int_destroy(&es); return -1;
+        }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
-        if (PyList_SetItem(list, (long)VECTOR(es)[i], item)) {
+        if (PyList_SetItem(list, VECTOR(es)[i], item)) {
           Py_DECREF(item);
-          igraph_vector_destroy(&es);
+          igraph_vector_int_destroy(&es);
           return -1;
         } /* PyList_SetItem stole a reference to the item automatically */
       }
-      igraph_vector_destroy(&es);
+      igraph_vector_int_destroy(&es);
     } else if (values != 0) {
       /* We don't have attributes with the given name yet. Create an entry
        * in the dict, create a new list, fill with None for vertices not in the
        * sequence and copy the rest */
-      long n2 = igraph_ecount(&gr->g);
+      igraph_integer_t n2 = igraph_ecount(&gr->g);
       list = PyList_New(n2);
-      if (list == 0) { igraph_vector_destroy(&es); return -1; }
-      for (i=0; i<n2; i++) {
-        Py_INCREF(Py_None);
-        PyList_SET_ITEM(list, i, Py_None);
+      if (list == 0) {
+        igraph_vector_int_destroy(&es);
+        return -1;
       }
-      for (i=0, j=0; i<no_of_edges; i++, j++) {
-        if (j == n) j = 0;
+      for (i = 0; i < n2; i++) {
+        Py_INCREF(Py_None);
+        if (PyList_SetItem(list, i, Py_None)) {
+          Py_DECREF(Py_None);
+          Py_DECREF(list);
+          return -1;
+        }
+      }
+      for (i = 0, j = 0; i < no_of_edges; i++, j++) {
+        if (j == n) {
+          j = 0;
+        }
         item = PySequence_GetItem(values, j);
         if (item == 0) {
-          igraph_vector_destroy(&es);
-          Py_DECREF(list); return -1;
+          igraph_vector_int_destroy(&es);
+          Py_DECREF(list);
+          return -1;
         }
         /* No need to Py_INCREF(item), PySequence_GetItem returns a new reference */
-        PyList_SET_ITEM(list, (long)VECTOR(es)[i], item);
+        if (PyList_SetItem(list, VECTOR(es)[i], item)) {
+          Py_DECREF(item);
+          Py_DECREF(list);
+          return -1;
+        }
         /* PyList_SET_ITEM stole a reference to the item automatically */
       }
-      igraph_vector_destroy(&es);
+      igraph_vector_int_destroy(&es);
       if (PyDict_SetItem(dict, attrname, list)) {
         Py_DECREF(list);
         return -1;
@@ -519,7 +634,7 @@ PyObject* igraphmodule_EdgeSeq_set_attribute_values(igraphmodule_EdgeSeqObject *
  */
 PyObject* igraphmodule_EdgeSeq_find(igraphmodule_EdgeSeqObject *self, PyObject *args) {
   PyObject *item;
-  long int i, n;
+  Py_ssize_t i, n;
 
   if (!PyArg_ParseTuple(args, "O", &item))
     return NULL;
@@ -528,7 +643,7 @@ PyObject* igraphmodule_EdgeSeq_find(igraphmodule_EdgeSeqObject *self, PyObject *
     /* Call the callable for every edge in the current sequence and return
      * the first one for which it evaluates to True */
     n = PySequence_Size((PyObject*)self);
-    for (i=0; i<n; i++) {
+    for (i = 0; i < n; i++) {
       PyObject *edge = PySequence_GetItem((PyObject*)self, i);
       PyObject *call_result;
       if (edge == 0)
@@ -548,7 +663,12 @@ PyObject* igraphmodule_EdgeSeq_find(igraphmodule_EdgeSeqObject *self, PyObject *
   } else if (PyLong_Check(item)) {
     /* Integers are interpreted as indices on the edge set and NOT on the
      * original, untouched edge sequence of the graph */
-    return PySequence_GetItem((PyObject*)self, PyLong_AsLong(item));
+    Py_ssize_t index = PyLong_AsSsize_t(item);
+    if (PyErr_Occurred()) {
+      return NULL;
+    } else {
+      return PySequence_GetItem((PyObject*)self, index);
+    }
   }
 
   PyErr_SetString(PyExc_IndexError, "no such edge");
@@ -564,19 +684,23 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
   igraphmodule_GraphObject *gr;
   igraph_integer_t igraph_idx;
   igraph_bool_t working_on_whole_graph = igraph_es_is_all(&self->es);
-  igraph_vector_t v, v2;
-  long i, j, n, m;
+  igraph_vector_int_t v, v2;
+  Py_ssize_t i, j, n, m;
 
   gr = self->gref;
   result = igraphmodule_EdgeSeq_copy(self);
-  if (result == 0)
+  if (result == 0) {
     return NULL;
+  }
 
   /* First, filter by positional arguments */
   n = PyTuple_Size(args);
   for (i = 0; i < n; i++) {
-    PyObject *item = PyTuple_GET_ITEM(args, i);
-    if (item == Py_None) {
+    PyObject *item = PyTuple_GetItem(args, i);
+    if (item == 0) {
+      Py_DECREF(result);
+      return NULL;
+    } else if (item == Py_None) {
       /* None means: select nothing */
       igraph_es_destroy(&result->es);
       igraph_es_none(&result->es);
@@ -586,9 +710,9 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
       /* Call the callable for every edge in the current sequence to
        * determine what's up */
       igraph_bool_t was_excluded = 0;
-      igraph_vector_t v;
+      igraph_vector_int_t v;
 
-      if (igraph_vector_init(&v, 0)) {
+      if (igraph_vector_int_init(&v, 0)) {
         igraphmodule_handle_igraph_error();
         return 0;
       }
@@ -599,18 +723,18 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
         PyObject *call_result;
         if (edge == 0) {
           Py_DECREF(result);
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           return NULL;
         }
         call_result = PyObject_CallFunctionObjArgs(item, edge, NULL);
         if (call_result == 0) {
           Py_DECREF(edge); Py_DECREF(result);
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           return NULL;
         }
         if (PyObject_IsTrue(call_result)) {
-          igraph_vector_push_back(&v,
-            igraphmodule_Edge_get_index_long((igraphmodule_EdgeObject*)edge));
+          igraph_vector_int_push_back(&v,
+            igraphmodule_Edge_get_index_as_igraph_integer((igraphmodule_EdgeObject*)edge));
         } else {
           was_excluded = 1;
         }
@@ -622,38 +746,38 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
         igraph_es_destroy(&result->es);
         if (igraph_es_vector_copy(&result->es, &v)) {
           Py_DECREF(result);
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           igraphmodule_handle_igraph_error();
           return NULL;
         }
       }
 
-      igraph_vector_destroy(&v);
+      igraph_vector_int_destroy(&v);
     } else if (PyLong_Check(item)) {
       /* Integers are treated specially: from now on, all remaining items
        * in the argument list must be integers and they will be used together
        * to restrict the edge set. Integers are interpreted as indices on the
        * edge set and NOT on the original, untouched edge sequence of the
        * graph */
-      if (igraph_vector_init(&v, 0)) {
+      if (igraph_vector_int_init(&v, 0)) {
         igraphmodule_handle_igraph_error();
         return 0;
       }
 
       if (!working_on_whole_graph) {
         /* Extract the current vertex sequence into a vector */
-        if (igraph_vector_init(&v2, 0)) {
-          igraph_vector_destroy(&v);
+        if (igraph_vector_int_init(&v2, 0)) {
+          igraph_vector_int_destroy(&v);
           igraphmodule_handle_igraph_error();
           return 0;
         }
         if (igraph_es_as_vector(&gr->g, self->es, &v2)) {
-          igraph_vector_destroy(&v);
-          igraph_vector_destroy(&v2);
+          igraph_vector_int_destroy(&v);
+          igraph_vector_int_destroy(&v2);
           igraphmodule_handle_igraph_error();
           return 0;
         }
-        m = igraph_vector_size(&v2);
+        m = igraph_vector_int_size(&v2);
       } else {
         /* v2 left uninitialized, we are not going to use it as it would
          * simply contain integers from 0 to ecount(g)-1 */
@@ -661,39 +785,56 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
       }
 
       for (; i < n; i++) {
-        PyObject *item2 = PyTuple_GET_ITEM(args, i);
-        long idx;
+        PyObject *item2 = PyTuple_GetItem(args, i);
+        igraph_integer_t idx;
+
+        if (item2 == 0) {
+          Py_DECREF(result);
+          igraph_vector_int_destroy(&v);
+          if (!working_on_whole_graph) {
+            igraph_vector_int_destroy(&v2);
+          }
+          return NULL;
+        }
         if (!PyLong_Check(item2)) {
           Py_DECREF(result);
           PyErr_SetString(PyExc_TypeError, "edge indices expected");
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           if (!working_on_whole_graph) {
-            igraph_vector_destroy(&v2);
+            igraph_vector_int_destroy(&v2);
+          }
+          return NULL;
+        }
+        if (igraphmodule_PyObject_to_integer_t(item2, &idx)) {
+          Py_DECREF(result);
+          igraph_vector_int_destroy(&v);
+          if (!working_on_whole_graph) {
+            igraph_vector_int_destroy(&v2);
           }
           return NULL;
         }
-        idx = PyLong_AsLong(item2);
         if (idx >= m || idx < 0) {
+          Py_DECREF(result);
           PyErr_SetString(PyExc_ValueError, "edge index out of range");
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           if (!working_on_whole_graph) {
-            igraph_vector_destroy(&v2);
+            igraph_vector_int_destroy(&v2);
           }
           return NULL;
         }
-        if (igraph_vector_push_back(&v, working_on_whole_graph ? idx : VECTOR(v2)[idx])) {
+        if (igraph_vector_int_push_back(&v, working_on_whole_graph ? idx : VECTOR(v2)[idx])) {
           Py_DECREF(result);
           igraphmodule_handle_igraph_error();
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           if (!working_on_whole_graph) {
-            igraph_vector_destroy(&v2);
+            igraph_vector_int_destroy(&v2);
           }
           return NULL;
         }
       }
 
       if (!working_on_whole_graph) {
-        igraph_vector_destroy(&v2);
+        igraph_vector_int_destroy(&v2);
       }
 
       igraph_es_destroy(&result->es);
@@ -701,34 +842,34 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
       if (igraph_es_vector_copy(&result->es, &v)) {
         Py_DECREF(result);
         igraphmodule_handle_igraph_error();
-        igraph_vector_destroy(&v);
+        igraph_vector_int_destroy(&v);
         return NULL;
       }
 
-      igraph_vector_destroy(&v);
+      igraph_vector_int_destroy(&v);
     } else {
       /* Iterators and everything that was not handled directly */
       PyObject *iter, *item2;
 
       /* Allocate stuff */
-      if (igraph_vector_init(&v, 0)) {
+      if (igraph_vector_int_init(&v, 0)) {
         igraphmodule_handle_igraph_error();
         return 0;
       }
       if (!working_on_whole_graph) {
         /* Extract the current vertex sequence into a vector */
-        if (igraph_vector_init(&v2, 0)) {
-          igraph_vector_destroy(&v);
+        if (igraph_vector_int_init(&v2, 0)) {
+          igraph_vector_int_destroy(&v);
           igraphmodule_handle_igraph_error();
           return 0;
         }
         if (igraph_es_as_vector(&gr->g, self->es, &v2)) {
-          igraph_vector_destroy(&v);
-          igraph_vector_destroy(&v2);
+          igraph_vector_int_destroy(&v);
+          igraph_vector_int_destroy(&v2);
           igraphmodule_handle_igraph_error();
           return 0;
         }
-        m = igraph_vector_size(&v2);
+        m = igraph_vector_int_size(&v2);
       } else {
         /* v2 left uninitialized, we are not going to use it as it would
          * simply contain integers from 0 to ecount(g)-1 */
@@ -753,9 +894,9 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
           ok = (iter != 0);
         }
         if (!ok) {
-          igraph_vector_destroy(&v);
+          igraph_vector_int_destroy(&v);
           if (!working_on_whole_graph) {
-            igraph_vector_destroy(&v2);
+            igraph_vector_int_destroy(&v2);
           }
           PyErr_SetString(PyExc_TypeError, "error while converting slice to iterator");
           Py_DECREF(result);
@@ -768,9 +909,9 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
 
       /* Did we manage to get an iterator? */
       if (iter == 0) {
-        igraph_vector_destroy(&v);
+        igraph_vector_int_destroy(&v);
         if (!working_on_whole_graph) {
-          igraph_vector_destroy(&v2);
+          igraph_vector_int_destroy(&v2);
         }
         PyErr_SetString(PyExc_TypeError, "invalid edge filter among positional arguments");
         Py_DECREF(result);
@@ -787,19 +928,19 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
             PyErr_SetString(PyExc_ValueError, "edge index out of range");
             Py_DECREF(result);
             Py_DECREF(iter);
-            igraph_vector_destroy(&v);
+            igraph_vector_int_destroy(&v);
             if (!working_on_whole_graph) {
-              igraph_vector_destroy(&v2);
+              igraph_vector_int_destroy(&v2);
             }
             return NULL;
           }
-          if (igraph_vector_push_back(&v, working_on_whole_graph ? igraph_idx : VECTOR(v2)[(long int) igraph_idx])) {
+          if (igraph_vector_int_push_back(&v, working_on_whole_graph ? igraph_idx : VECTOR(v2)[igraph_idx])) {
             Py_DECREF(result);
             Py_DECREF(iter);
             igraphmodule_handle_igraph_error();
-            igraph_vector_destroy(&v);
+            igraph_vector_int_destroy(&v);
             if (!working_on_whole_graph) {
-              igraph_vector_destroy(&v2);
+              igraph_vector_int_destroy(&v2);
             }
             return NULL;
           }
@@ -808,12 +949,12 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
 
       /* Deallocate stuff */
       if (!working_on_whole_graph) {
-        igraph_vector_destroy(&v2);
+        igraph_vector_int_destroy(&v2);
       }
 
       Py_DECREF(iter);
       if (PyErr_Occurred()) {
-        igraph_vector_destroy(&v);
+        igraph_vector_int_destroy(&v);
         Py_DECREF(result);
         return 0;
       }
@@ -823,10 +964,10 @@ PyObject* igraphmodule_EdgeSeq_select(igraphmodule_EdgeSeqObject *self, PyObject
       if (igraph_es_vector_copy(&result->es, &v)) {
         Py_DECREF(result);
         igraphmodule_handle_igraph_error();
-        igraph_vector_destroy(&v);
+        igraph_vector_int_destroy(&v);
         return NULL;
       }
-      igraph_vector_destroy(&v);
+      igraph_vector_int_destroy(&v);
     }
   }
 
@@ -876,39 +1017,6 @@ PyMethodDef igraphmodule_EdgeSeq_methods[] = {
   {NULL}
 };
 
-/**
- * \ingroup python_interface_edgeseq
- * This is the collection of functions necessary to implement the
- * edge sequence as a real sequence (e.g. allowing to reference
- * edges by indices)
- */
-static PySequenceMethods igraphmodule_EdgeSeq_as_sequence = {
-  (lenfunc)igraphmodule_EdgeSeq_sq_length,
-  0,               /* sq_concat */
-  0,               /* sq_repeat */
-  (ssizeargfunc)igraphmodule_EdgeSeq_sq_item, /* sq_item */
-  0,                                          /* sq_slice */
-  0,                                          /* sq_ass_item */
-  0,                                          /* sq_ass_slice */
-  0,                                          /* sq_contains */
-  0,                                          /* sq_inplace_concat */
-  0,                                          /* sq_inplace_repeat */
-};
-
-/**
- * \ingroup python_interface_edgeseq
- * This is the collection of functions necessary to implement the
- * edge sequence as a mapping (which maps attribute names to values)
- */
-static PyMappingMethods igraphmodule_EdgeSeq_as_mapping = {
-    /* returns the number of edge attributes */
-    (lenfunc) 0,
-    /* returns the values of an attribute by name */
-    (binaryfunc) igraphmodule_EdgeSeq_get_attribute_values_mapping,
-    /* sets the values of an attribute by name */
-    (objobjargproc) igraphmodule_EdgeSeq_set_attribute_values_mapping,
-};
-
 /**
  * \ingroup python_interface_edgeseq
  * Returns the graph where the edge sequence belongs
@@ -926,21 +1034,21 @@ PyObject* igraphmodule_EdgeSeq_get_graph(igraphmodule_EdgeSeqObject* self,
 PyObject* igraphmodule_EdgeSeq_get_indices(igraphmodule_EdgeSeqObject* self,
   void* closure) {
   igraphmodule_GraphObject *gr = self->gref;
-  igraph_vector_t es;
+  igraph_vector_int_t es;
   PyObject *result;
 
-  if (igraph_vector_init(&es, 0)) {
+  if (igraph_vector_int_init(&es, 0)) {
     igraphmodule_handle_igraph_error();
     return 0;
   }
   if (igraph_es_as_vector(&gr->g, self->es, &es)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&es);
+    igraph_vector_int_destroy(&es);
     return 0;
   }
 
-  result = igraphmodule_vector_t_to_PyList(&es, IGRAPHMODULE_TYPE_INT);
-  igraph_vector_destroy(&es);
+  result = igraphmodule_vector_int_t_to_PyList(&es);
+  igraph_vector_int_destroy(&es);
 
   return result;
 }
@@ -958,57 +1066,48 @@ PyGetSetDef igraphmodule_EdgeSeq_getseters[] = {
   {NULL}
 };
 
-/** \ingroup python_interface_edgeseq
- * Python type object referencing the methods Python calls when it performs various operations on
- * an edge sequence of a graph
+/**
+ * \ingroup python_interface_edgeseq
+ * Member table for the \c igraph.EdgeSeq object
  */
-PyTypeObject igraphmodule_EdgeSeqType =
-{
-  PyVarObject_HEAD_INIT(0, 0)
-  "igraph._igraph.EdgeSeq",                 /* tp_name */
-  sizeof(igraphmodule_EdgeSeqObject),       /* tp_basicsize */
-  0,                                        /* tp_itemsize */
-  (destructor)igraphmodule_EdgeSeq_dealloc, /* tp_dealloc */
-  0,                                        /* tp_print */
-  0,                                        /* tp_getattr */
-  0,                                        /* tp_setattr */
-  0,                                        /* tp_compare (2.x) / tp_reserved (3.x) */
-  0,                                        /* tp_repr */
-  0,                                        /* tp_as_number */
-  &igraphmodule_EdgeSeq_as_sequence,        /* tp_as_sequence */
-  &igraphmodule_EdgeSeq_as_mapping,         /* tp_as_mapping */
-  0,                                        /* tp_hash */
-  0,                                        /* tp_call */
-  0,                                        /* tp_str */
-  0,                                        /* tp_getattro */
-  0,                                        /* tp_setattro */
-  0,                                        /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /* tp_flags */
+PyMemberDef igraphmodule_EdgeSeq_members[] = {
+  {"__weaklistoffset__", T_PYSSIZET, offsetof(igraphmodule_EdgeSeqObject, weakreflist), READONLY},
+  { 0 }
+};
+
+PyDoc_STRVAR(
+  igraphmodule_EdgeSeq_doc,
   "Low-level representation of an edge sequence.\n\n" /* tp_doc */
   "Don't use it directly, use L{igraph.EdgeSeq} instead.\n\n"
-  "@deffield ref: Reference",
-  0,                                          /* tp_traverse */
-  0,                                          /* tp_clear */
-  0,                                          /* tp_richcompare */
-  offsetof(igraphmodule_EdgeSeqObject, weakreflist), /* tp_weaklistoffset */
-  0,                                          /* tp_iter */
-  0,                                          /* tp_iternext */
-  igraphmodule_EdgeSeq_methods,               /* tp_methods */
-  0,                                          /* tp_members */
-  igraphmodule_EdgeSeq_getseters,             /* tp_getset */
-  0,                                          /* tp_base */
-  0,                                          /* tp_dict */
-  0,                                          /* tp_descr_get */
-  0,                                          /* tp_descr_set */
-  0,                                          /* tp_dictoffset */
-  (initproc) igraphmodule_EdgeSeq_init,       /* tp_init */
-  0,                                          /* tp_alloc */
-  (newfunc) igraphmodule_EdgeSeq_new,         /* tp_new */
-  0,                                          /* tp_free */
-  0,                                          /* tp_is_gc */
-  0,                                          /* tp_bases */
-  0,                                          /* tp_mro */
-  0,                                          /* tp_cache  */
-  0,                                          /* tp_subclasses */
-  0,                                          /* tp_weakreflist */
-};
+  "@deffield ref: Reference"
+);
+
+int igraphmodule_EdgeSeq_register_type() {
+  PyType_Slot slots[] = {
+    { Py_tp_init, igraphmodule_EdgeSeq_init },
+    { Py_tp_dealloc, igraphmodule_EdgeSeq_dealloc },
+    { Py_tp_members, igraphmodule_EdgeSeq_members },
+    { Py_tp_methods, igraphmodule_EdgeSeq_methods },
+    { Py_tp_getset, igraphmodule_EdgeSeq_getseters },
+    { Py_tp_doc, (void*) igraphmodule_EdgeSeq_doc },
+
+    { Py_sq_length, igraphmodule_EdgeSeq_sq_length },
+    { Py_sq_item, igraphmodule_EdgeSeq_sq_item },
+
+    { Py_mp_subscript, igraphmodule_EdgeSeq_get_attribute_values_mapping },
+    { Py_mp_ass_subscript, igraphmodule_EdgeSeq_set_attribute_values_mapping },
+  
+    { 0 }
+  };
+
+  PyType_Spec spec = {
+    "igraph._igraph.EdgeSeq",                   /* name */
+    sizeof(igraphmodule_EdgeSeqObject),         /* basicsize */
+    0,                                          /* itemsize */
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,   /* flags */
+    slots,                                      /* slots */
+  };
+
+  igraphmodule_EdgeSeqType = (PyTypeObject*) PyType_FromSpec(&spec);
+  return igraphmodule_EdgeSeqType == 0;
+}
