@@ -33,6 +33,7 @@
 #include "edgeseqobject.h"
 #include "error.h"
 #include "graphobject.h"
+#include "pyhelpers.h"
 #include "random.h"
 #include "vertexobject.h"
 #include "vertexseqobject.h"
@@ -141,7 +142,7 @@ static int igraphmodule_clear(PyObject *m) {
   return 0;
 }
 
-static int igraphmodule_igraph_interrupt_hook(void* data) {
+static igraph_error_t igraphmodule_igraph_interrupt_hook(void* data) {
   if (PyErr_CheckSignals()) {
     IGRAPH_FINALLY_FREE();
     return IGRAPH_INTERRUPTED;
@@ -149,26 +150,26 @@ static int igraphmodule_igraph_interrupt_hook(void* data) {
   return IGRAPH_SUCCESS;
 }
 
-int igraphmodule_igraph_progress_hook(const char* message, igraph_real_t percent,
+igraph_error_t igraphmodule_igraph_progress_hook(const char* message, igraph_real_t percent,
 				       void* data) {
   PyObject* progress_handler = GETSTATE(0)->progress_handler;
+  PyObject *result;
 
   if (progress_handler) {
-    PyObject *result;
     if (PyCallable_Check(progress_handler)) {
-      result=PyObject_CallFunction(progress_handler,
-				   "sd", message, (double)percent);
-      if (result)
+      result = PyObject_CallFunction(progress_handler, "sd", message, (double)percent);
+      if (result) {
         Py_DECREF(result);
-      else
+      } else {
         return IGRAPH_INTERRUPTED;
+      }
     }
   }
   
   return IGRAPH_SUCCESS;
 }
 
-int igraphmodule_igraph_status_hook(const char* message, void*data) {
+igraph_error_t igraphmodule_igraph_status_hook(const char* message, void*data) {
   PyObject* status_handler = GETSTATE(0)->status_handler;
 
   if (status_handler) {
@@ -199,7 +200,8 @@ PyObject* igraphmodule_set_progress_handler(PyObject* self, PyObject* o) {
 
   Py_XDECREF(progress_handler);
   if (o == Py_None)
-    o = 0;
+    o = NULL;
+
   Py_XINCREF(o);
   GETSTATE(self)->progress_handler=o;
 
@@ -219,9 +221,11 @@ PyObject* igraphmodule_set_status_handler(PyObject* self, PyObject* o) {
     Py_RETURN_NONE;
 
   Py_XDECREF(status_handler);
-  if (o == Py_None)
-    o = 0;
-  Py_INCREF(o);
+  if (o == Py_None) {
+    o = NULL;
+  }
+
+  Py_XINCREF(o);
   GETSTATE(self)->status_handler = o;
 
   Py_RETURN_NONE;
@@ -229,68 +233,86 @@ PyObject* igraphmodule_set_status_handler(PyObject* self, PyObject* o) {
 
 PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwds) {
   static char* kwlist[] = {"vs", "coords", NULL};
-  PyObject *vs, *o, *o1=0, *o2=0, *coords = Py_False;
+  PyObject *vs, *o, *o1 = 0, *o2 = 0, *o1_float, *o2_float, *coords = Py_False;
   igraph_matrix_t mtrx;
-  igraph_vector_t result;
+  igraph_vector_int_t result;
   igraph_matrix_t resmat;
-  long no_of_nodes, i;
+  Py_ssize_t no_of_nodes, i;
   
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O", kwlist, &PyList_Type, &vs, &coords))
     return NULL;
   
-  no_of_nodes=PyList_Size(vs);
+  no_of_nodes = PyList_Size(vs);
   if (igraph_matrix_init(&mtrx, no_of_nodes, 2)) {
     igraphmodule_handle_igraph_error();
     return NULL;
   }
-  for (i=0; i<no_of_nodes; i++) {
-    o=PyList_GetItem(vs, i);
-    if (PyList_Check(o)) {
-      if (PyList_Size(o) >= 2) {
-        o1=PyList_GetItem(o, 0);
-        o2=PyList_GetItem(o, 1);
-        if (PyList_Size(o) > 2)
-          PyErr_Warn(PyExc_Warning, "vertex with more than 2 coordinates found, considering only the first 2");
+
+  for (i = 0; i < no_of_nodes; i++) {
+    o = PyList_GetItem(vs, i);
+
+    if (PySequence_Check(o)) {
+      if (PySequence_Size(o) >= 2) {
+        o1 = PySequence_GetItem(o, 0);
+        if (!o1) {
+          igraph_matrix_destroy(&mtrx);
+          return NULL;
+        }
+
+        o2 = PySequence_GetItem(o, 1);
+        if (!o2) {
+          Py_DECREF(o1);
+          igraph_matrix_destroy(&mtrx);
+          return NULL;
+        }
+
+        if (PySequence_Size(o) > 2) {
+          PY_IGRAPH_WARN("vertex with more than 2 coordinates found, considering only the first 2");
+        }
       } else {
         PyErr_SetString(PyExc_TypeError, "vertex with less than 2 coordinates found");
         igraph_matrix_destroy(&mtrx);
-	    return NULL;
-      }
-    } else if (PyTuple_Check(o)) {
-      if (PyTuple_Size(o) >= 2) {
-	    o1=PyTuple_GetItem(o, 0);
-	    o2=PyTuple_GetItem(o, 1);
-	    if (PyTuple_Size(o) > 2)
-	      PyErr_Warn(PyExc_Warning, "vertex with more than 2 coordinates found, considering only the first 2");
-       } else {
-	      PyErr_SetString(PyExc_TypeError, "vertex with less than 2 coordinates found");
-	    igraph_matrix_destroy(&mtrx);
-	    return NULL;
+        return NULL;
       }
+    } else {
+      PyErr_SetString(PyExc_TypeError, "convex_hull() must receive a list of indexable sequences");
+      igraph_matrix_destroy(&mtrx);
+      return NULL;
     }
     
     if (!PyNumber_Check(o1) || !PyNumber_Check(o2)) {
       PyErr_SetString(PyExc_TypeError, "vertex coordinates must be numeric");
+      Py_DECREF(o2);
+      Py_DECREF(o1);
       igraph_matrix_destroy(&mtrx);
       return NULL;
     }
-    /* o, o1 and o2 were borrowed, but now o1 and o2 are actual references! */
-    o1=PyNumber_Float(o1); o2=PyNumber_Float(o2);
-    if (!o1 || !o2) {
-      PyErr_SetString(PyExc_TypeError, "vertex coordinate conversion to float failed");
-      Py_XDECREF(o1);
-      Py_XDECREF(o2);
+
+    o1_float = PyNumber_Float(o1);
+    if (!o1_float) {
+      Py_DECREF(o2);
+      Py_DECREF(o1);
       igraph_matrix_destroy(&mtrx);
       return NULL;
     }
-    MATRIX(mtrx, i, 0)=(igraph_real_t)PyFloat_AsDouble(o1);
-    MATRIX(mtrx, i, 1)=(igraph_real_t)PyFloat_AsDouble(o2);
     Py_DECREF(o1);
+
+    o2_float = PyNumber_Float(o2);
+    if (!o2_float) {
+      Py_DECREF(o2);
+      igraph_matrix_destroy(&mtrx);
+      return NULL;
+    }
     Py_DECREF(o2);
+
+    MATRIX(mtrx, i, 0) = PyFloat_AsDouble(o1_float);
+    MATRIX(mtrx, i, 1) = PyFloat_AsDouble(o2_float);
+    Py_DECREF(o1_float);
+    Py_DECREF(o2_float);
   }
 
   if (!PyObject_IsTrue(coords)) {
-    if (igraph_vector_init(&result, 0)) {
+    if (igraph_vector_int_init(&result, 0)) {
       igraphmodule_handle_igraph_error();
       igraph_matrix_destroy(&mtrx);
       return NULL;
@@ -298,11 +320,11 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
     if (igraph_convex_hull(&mtrx, &result, 0)) {
       igraphmodule_handle_igraph_error();
       igraph_matrix_destroy(&mtrx);
-      igraph_vector_destroy(&result);
+      igraph_vector_int_destroy(&result);
       return NULL;
     }    
-    o=igraphmodule_vector_t_to_PyList(&result, IGRAPHMODULE_TYPE_INT);
-    igraph_vector_destroy(&result);
+    o = igraphmodule_vector_int_t_to_PyList(&result);
+    igraph_vector_int_destroy(&result);
   } else {
     if (igraph_matrix_init(&resmat, 0, 0)) {
       igraphmodule_handle_igraph_error();
@@ -315,7 +337,7 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
       igraph_matrix_destroy(&resmat);
       return NULL;
     }        
-    o=igraphmodule_matrix_t_to_PyList(&resmat, IGRAPHMODULE_TYPE_FLOAT);
+    o = igraphmodule_matrix_t_to_PyList(&resmat, IGRAPHMODULE_TYPE_FLOAT);
     igraph_matrix_destroy(&resmat);
   }
   
@@ -329,42 +351,44 @@ PyObject* igraphmodule_community_to_membership(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "merges", "nodes", "steps", "return_csize", NULL };
   PyObject *merges_o, *return_csize = Py_False, *result_o;
-  igraph_matrix_t merges;
-  igraph_vector_t result, csize, *csize_p = 0;
-  long int nodes, steps;
+  igraph_matrix_int_t merges;
+  igraph_vector_int_t result, csize, *csize_p = 0;
+  Py_ssize_t nodes, steps;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!ll|O", kwlist,
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!nn|O", kwlist,
       &PyList_Type, &merges_o, &nodes, &steps, &return_csize)) return NULL;
 
-  if (igraphmodule_PyList_to_matrix_t_with_minimum_column_count(merges_o, &merges, 2)) return NULL;
+  if (igraphmodule_PyList_to_matrix_int_t_with_minimum_column_count(merges_o, &merges, 2)) return NULL;
 
-  if (igraph_vector_init(&result, nodes)) {
+  CHECK_SSIZE_T_RANGE(nodes, "number of nodes");
+  CHECK_SSIZE_T_RANGE(steps, "number of steps");
+
+  if (igraph_vector_int_init(&result, nodes)) {
     igraphmodule_handle_igraph_error();
-    igraph_matrix_destroy(&merges);
+    igraph_matrix_int_destroy(&merges);
     return NULL;
   }
 
   if (PyObject_IsTrue(return_csize)) {
-    igraph_vector_init(&csize, 0);
+    igraph_vector_int_init(&csize, 0);
     csize_p = &csize;
   }
 
-  if (igraph_community_to_membership(&merges, (igraph_integer_t)nodes,
-        (igraph_integer_t)steps, &result, csize_p)) {
+  if (igraph_community_to_membership(&merges, nodes, steps, &result, csize_p)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&result);
-    if (csize_p) igraph_vector_destroy(csize_p);
-    igraph_matrix_destroy(&merges);
+    igraph_vector_int_destroy(&result);
+    if (csize_p) igraph_vector_int_destroy(csize_p);
+    igraph_matrix_int_destroy(&merges);
     return NULL;
   }
-  igraph_matrix_destroy(&merges);
+  igraph_matrix_int_destroy(&merges);
 
-  result_o = igraphmodule_vector_t_to_PyList(&result, IGRAPHMODULE_TYPE_INT);
-  igraph_vector_destroy(&result);
+  result_o = igraphmodule_vector_int_t_to_PyList(&result);
+  igraph_vector_int_destroy(&result);
 
   if (csize_p) {
-	PyObject* csize_o = igraphmodule_vector_t_to_PyList(csize_p, IGRAPHMODULE_TYPE_INT);
-	igraph_vector_destroy(csize_p);
+	PyObject* csize_o = igraphmodule_vector_int_t_to_PyList(csize_p);
+	igraph_vector_int_destroy(csize_p);
 	if (csize_o) return Py_BuildValue("NN", result_o, csize_o);
 	Py_DECREF(result_o);
 	return NULL;
@@ -378,34 +402,39 @@ PyObject* igraphmodule_compare_communities(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "comm1", "comm2", "method", NULL };
   PyObject *comm1_o, *comm2_o, *method_o = Py_None;
-  igraph_vector_t comm1, comm2;
+  igraph_vector_int_t comm1, comm2;
   igraph_community_comparison_t method = IGRAPH_COMMCMP_VI;
   igraph_real_t result;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|O", kwlist,
-      &comm1_o, &comm2_o, &method_o))
+      &comm1_o, &comm2_o, &method_o)) {
     return NULL;
+  }
 
-  if (igraphmodule_PyObject_to_community_comparison_t(method_o, &method))
+  if (igraphmodule_PyObject_to_community_comparison_t(method_o, &method)) {
     return NULL;
+  }
 
-  if (igraphmodule_PyObject_to_vector_t(comm1_o, &comm1, 0))
+  if (igraphmodule_PyObject_to_vector_int_t(comm1_o, &comm1)) {
     return NULL;
-  if (igraphmodule_PyObject_to_vector_t(comm2_o, &comm2, 0)) {
-    igraph_vector_destroy(&comm1);
+  }
+
+  if (igraphmodule_PyObject_to_vector_int_t(comm2_o, &comm2)) {
+    igraph_vector_int_destroy(&comm1);
     return NULL;
   }
 
   if (igraph_compare_communities(&comm1, &comm2, &result, method)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&comm1);
-    igraph_vector_destroy(&comm2);
+    igraph_vector_int_destroy(&comm1);
+    igraph_vector_int_destroy(&comm2);
     return NULL;
   }
-  igraph_vector_destroy(&comm1);
-  igraph_vector_destroy(&comm2);
 
-  return PyFloat_FromDouble((double)result);
+  igraph_vector_int_destroy(&comm1);
+  igraph_vector_int_destroy(&comm2);
+
+  return igraphmodule_real_t_to_PyObject(result, IGRAPHMODULE_TYPE_FLOAT);
 }
 
 
@@ -413,7 +442,7 @@ PyObject* igraphmodule_is_degree_sequence(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "out_deg", "in_deg", NULL };
   PyObject *out_deg_o = 0, *in_deg_o = 0;
-  igraph_vector_t out_deg, in_deg;
+  igraph_vector_int_t out_deg, in_deg;
   igraph_bool_t is_directed, result;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist,
@@ -422,25 +451,25 @@ PyObject* igraphmodule_is_degree_sequence(PyObject *self,
 
   is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
 
-  if (igraphmodule_PyObject_to_vector_t(out_deg_o, &out_deg, 0))
+  if (igraphmodule_PyObject_to_vector_int_t(out_deg_o, &out_deg))
     return NULL;
 
-  if (is_directed && igraphmodule_PyObject_to_vector_t(in_deg_o, &in_deg, 0)) {
-    igraph_vector_destroy(&out_deg);
+  if (is_directed && igraphmodule_PyObject_to_vector_int_t(in_deg_o, &in_deg)) {
+    igraph_vector_int_destroy(&out_deg);
     return NULL;
   }
 
   if (igraph_is_graphical(&out_deg, is_directed ? &in_deg : 0, IGRAPH_LOOPS_SW | IGRAPH_MULTI_SW, &result)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&out_deg);
+    igraph_vector_int_destroy(&out_deg);
     if (is_directed)
-      igraph_vector_destroy(&in_deg);
+      igraph_vector_int_destroy(&in_deg);
     return NULL;
   }
 
-  igraph_vector_destroy(&out_deg);
+  igraph_vector_int_destroy(&out_deg);
   if (is_directed)
-    igraph_vector_destroy(&in_deg);
+    igraph_vector_int_destroy(&in_deg);
 
   if (result)
     Py_RETURN_TRUE;
@@ -453,7 +482,7 @@ PyObject* igraphmodule_is_graphical_degree_sequence(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "out_deg", "in_deg", NULL };
   PyObject *out_deg_o = 0, *in_deg_o = 0;
-  igraph_vector_t out_deg, in_deg;
+  igraph_vector_int_t out_deg, in_deg;
   igraph_bool_t is_directed, result;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist,
@@ -462,25 +491,25 @@ PyObject* igraphmodule_is_graphical_degree_sequence(PyObject *self,
 
   is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
 
-  if (igraphmodule_PyObject_to_vector_t(out_deg_o, &out_deg, 0))
+  if (igraphmodule_PyObject_to_vector_int_t(out_deg_o, &out_deg))
     return NULL;
 
-  if (is_directed && igraphmodule_PyObject_to_vector_t(in_deg_o, &in_deg, 0)) {
-    igraph_vector_destroy(&out_deg);
+  if (is_directed && igraphmodule_PyObject_to_vector_int_t(in_deg_o, &in_deg)) {
+    igraph_vector_int_destroy(&out_deg);
     return NULL;
   }
 
   if (igraph_is_graphical(&out_deg, is_directed ? &in_deg : 0, IGRAPH_SIMPLE_SW, &result)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&out_deg);
+    igraph_vector_int_destroy(&out_deg);
     if (is_directed)
-      igraph_vector_destroy(&in_deg);
+      igraph_vector_int_destroy(&in_deg);
     return NULL;
   }
 
-  igraph_vector_destroy(&out_deg);
+  igraph_vector_int_destroy(&out_deg);
   if (is_directed)
-    igraph_vector_destroy(&in_deg);
+    igraph_vector_int_destroy(&in_deg);
 
   if (result)
     Py_RETURN_TRUE;
@@ -493,7 +522,7 @@ PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kw
   static char* kwlist[] = { "out_deg", "in_deg", "loops", "multiple", NULL };
   PyObject *out_deg_o = 0, *in_deg_o = 0;
   PyObject *loops = Py_False, *multiple = Py_False;
-  igraph_vector_t out_deg, in_deg;
+  igraph_vector_int_t out_deg, in_deg;
   igraph_bool_t is_directed, result;
   int allowed_edge_types;
 
@@ -503,11 +532,11 @@ PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kw
 
   is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
 
-  if (igraphmodule_PyObject_to_vector_t(out_deg_o, &out_deg, 0))
+  if (igraphmodule_PyObject_to_vector_int_t(out_deg_o, &out_deg))
     return NULL;
 
-  if (is_directed && igraphmodule_PyObject_to_vector_t(in_deg_o, &in_deg, 0)) {
-    igraph_vector_destroy(&out_deg);
+  if (is_directed && igraphmodule_PyObject_to_vector_int_t(in_deg_o, &in_deg)) {
+    igraph_vector_int_destroy(&out_deg);
     return NULL;
   }
 
@@ -521,16 +550,16 @@ PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kw
 
   if (igraph_is_graphical(&out_deg, is_directed ? &in_deg : 0, allowed_edge_types, &result)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&out_deg);
+    igraph_vector_int_destroy(&out_deg);
     if (is_directed) {
-      igraph_vector_destroy(&in_deg);
+      igraph_vector_int_destroy(&in_deg);
     }
     return NULL;
   }
 
-  igraph_vector_destroy(&out_deg);
+  igraph_vector_int_destroy(&out_deg);
   if (is_directed) {
-    igraph_vector_destroy(&in_deg);
+    igraph_vector_int_destroy(&in_deg);
   }
 
   if (result)
@@ -541,14 +570,15 @@ PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kw
 
 
 PyObject* igraphmodule_power_law_fit(PyObject *self, PyObject *args, PyObject *kwds) {
-  static char* kwlist[] = { "data", "xmin", "force_continuous", NULL };
+  static char* kwlist[] = { "data", "xmin", "force_continuous", "p_precision", NULL };
   PyObject *data_o, *force_continuous_o = Py_False;
   igraph_vector_t data;
   igraph_plfit_result_t result;
-  double xmin = -1;
+  double xmin = -1, p_precision = 0.01;
+  igraph_real_t p;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|dO", kwlist, &data_o,
-        &xmin, &force_continuous_o))
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|dOd", kwlist, &data_o,
+        &xmin, &force_continuous_o, &p_precision))
     return NULL;
 
   if (igraphmodule_PyObject_float_to_vector_t(data_o, &data))
@@ -560,40 +590,99 @@ PyObject* igraphmodule_power_law_fit(PyObject *self, PyObject *args, PyObject *k
     return NULL;
   }
 
+  if (igraph_plfit_result_calculate_p_value(&result, &p, p_precision)) {
+    igraphmodule_handle_igraph_error();
+    igraph_vector_destroy(&data);
+    return NULL;
+  }
+
   igraph_vector_destroy(&data);
 
   return Py_BuildValue("Oddddd", result.continuous ? Py_True : Py_False,
-      result.alpha, result.xmin, result.L, result.D, result.p);
+      result.alpha, result.xmin, result.L, result.D, (double) p);
 }
 
 PyObject* igraphmodule_split_join_distance(PyObject *self,
   PyObject *args, PyObject *kwds) {
   static char* kwlist[] = { "comm1", "comm2", NULL };
   PyObject *comm1_o, *comm2_o;
-  igraph_vector_t comm1, comm2;
+  igraph_vector_int_t comm1, comm2;
   igraph_integer_t distance12, distance21;
 
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO", kwlist,
       &comm1_o, &comm2_o))
     return NULL;
 
-  if (igraphmodule_PyObject_to_vector_t(comm1_o, &comm1, 0))
+  if (igraphmodule_PyObject_to_vector_int_t(comm1_o, &comm1)) {
     return NULL;
-  if (igraphmodule_PyObject_to_vector_t(comm2_o, &comm2, 0)) {
-    igraph_vector_destroy(&comm1);
+  }
+
+  if (igraphmodule_PyObject_to_vector_int_t(comm2_o, &comm2)) {
+    igraph_vector_int_destroy(&comm1);
     return NULL;
   }
 
   if (igraph_split_join_distance(&comm1, &comm2, &distance12, &distance21)) {
     igraphmodule_handle_igraph_error();
-    igraph_vector_destroy(&comm1);
-    igraph_vector_destroy(&comm2);
+    igraph_vector_int_destroy(&comm1);
+    igraph_vector_int_destroy(&comm2);
     return NULL;
   }
-  igraph_vector_destroy(&comm1);
-  igraph_vector_destroy(&comm2);
 
-  return Py_BuildValue("ll", (long)distance12, (long)distance21);
+  igraph_vector_int_destroy(&comm1);
+  igraph_vector_int_destroy(&comm2);
+
+  /* sizeof(Py_ssize_t) is most likely the same as sizeof(igraph_integer_t),
+   * but even if it isn't, we cast explicitly so we are safe */
+  return Py_BuildValue("nn", (Py_ssize_t)distance12, (Py_ssize_t)distance21);
+}
+
+#define LOCALE_CAPSULE_TYPE "igraph._igraph.locale_capsule"
+
+void igraphmodule__destroy_locale_capsule(PyObject *capsule) {
+  igraph_safelocale_t* loc = (igraph_safelocale_t*) PyCapsule_GetPointer(capsule, LOCALE_CAPSULE_TYPE);
+  if (loc) {
+    PyMem_Free(loc);
+  }
+}
+
+PyObject* igraphmodule__enter_safelocale(PyObject* self) {
+  igraph_safelocale_t* loc;
+  PyObject* capsule;
+
+  loc = PyMem_Malloc(sizeof(loc));
+  if (loc == NULL) {
+    PyErr_NoMemory();
+    return NULL;
+  }
+
+  capsule = PyCapsule_New(loc, LOCALE_CAPSULE_TYPE, igraphmodule__destroy_locale_capsule);
+  if (capsule == NULL) {
+    return NULL;
+  }
+
+  if (igraph_enter_safelocale(loc)) {
+    Py_DECREF(capsule);
+    igraphmodule_handle_igraph_error();
+  }
+
+  return capsule;
+}
+
+PyObject* igraphmodule__exit_safelocale(PyObject *self, PyObject *capsule) {
+  igraph_safelocale_t* loc;
+
+  if (!PyCapsule_IsValid(capsule, LOCALE_CAPSULE_TYPE)) {
+    PyErr_SetString(PyExc_TypeError, "expected locale capsule");
+    return NULL;
+  }
+
+  loc = (igraph_safelocale_t*) PyCapsule_GetPointer(capsule, LOCALE_CAPSULE_TYPE);
+  if (loc != NULL) {
+    igraph_exit_safelocale(loc);
+  }
+
+  Py_RETURN_NONE;
 }
 
 /** \ingroup python_interface
@@ -611,7 +700,7 @@ static PyMethodDef igraphmodule_methods[] =
   },
   {"_power_law_fit", (PyCFunction)igraphmodule_power_law_fit,
     METH_VARARGS | METH_KEYWORDS,
-    "_power_law_fit(data, xmin=-1, force_continuous=False)\n--\n\n"
+    "_power_law_fit(data, xmin=-1, force_continuous=False, p_precision=0.01)\n--\n\n"
   },
   {"convex_hull", (PyCFunction)igraphmodule_convex_hull,
     METH_VARARGS | METH_KEYWORDS,
@@ -690,12 +779,15 @@ static PyMethodDef igraphmodule_methods[] =
       "  with at least three attributes: C{random}, C{randint} and C{gauss}.\n"
       "  Each of them must be callable and their signature and behaviour\n"
       "  must be identical to C{random.random}, C{random.randint} and\n"
-      "  C{random.gauss}. By default, igraph uses the C{random} module for\n"
-      "  random number generation, but you can supply your alternative\n"
-      "  implementation here. If the given generator is C{None}, igraph\n"
-      "  reverts to the default Mersenne twister generator implemented in the\n"
-      "  C layer, which might be slightly faster than calling back to Python\n"
-      "  for random numbers, but you cannot set its seed or save its state.\n"
+      "  C{random.gauss}. Optionally, the object can provide a function named\n"
+      "  C{getrandbits} with a signature identical to C{randpm.getrandbits}\n"
+      "  that provides a given number of random bits on demand. By default,\n"
+      "  igraph uses the C{random} module for random number generation, but\n"
+      "  you can supply your alternative implementation here. If the given\n"
+      "  generator is C{None}, igraph reverts to the default PCG32 generator\n"
+      "  implemented in the C layer, which might be slightly faster than\n"
+      "  calling back to Python for random numbers, but you cannot set its\n"
+      "  seed or save its state.\n"
   },
   {"set_status_handler", igraphmodule_set_status_handler, METH_O,
       "set_status_handler(handler)\n--\n\n"
@@ -724,6 +816,18 @@ static PyMethodDef igraphmodule_methods[] =
     METH_VARARGS | METH_KEYWORDS,
     "_intersection(graphs, edgemaps)\n--\n\n"
   },
+  {"_enter_safelocale", (PyCFunction)igraphmodule__enter_safelocale,
+    METH_NOARGS,
+    "_enter_safelocale()\n--\n\n"
+    "Helper function for the L{safe_locale()} context manager. Do not use\n"
+    "directly in your own code."
+  },
+  {"_exit_safelocale", (PyCFunction)igraphmodule__exit_safelocale,
+    METH_O,
+    "_exit_safelocale(locale)\n--\n\n"
+    "Helper function for the L{safe_locale()} context manager. Do not use\n"
+    "directly in your own code."
+  },
   {NULL, NULL, 0, NULL}
 };
 
@@ -798,6 +902,9 @@ PyObject* PyInit__igraph(void)
   static void *PyIGraph_API[PyIGraph_API_pointers];
   PyObject *c_api_object;
 
+  /* Prevent linking 64-bit igraph to 32-bit Python */
+  PY_IGRAPH_ASSERT_AT_BUILD_TIME(sizeof(igraph_integer_t) >= sizeof(Py_ssize_t));
+
   /* Check if the module is already initialized (possibly in another Python
    * interpreter. If so, bail out as we don't support this. */
   if (igraphmodule_initialized) {
@@ -806,30 +913,24 @@ PyObject* PyInit__igraph(void)
     INITERROR;
   }
 
-  /* Initialize VertexSeq, EdgeSeq */
-  if (PyType_Ready(&igraphmodule_VertexSeqType) < 0)
-    INITERROR;
-  if (PyType_Ready(&igraphmodule_EdgeSeqType) < 0)
-    INITERROR;
-  
-  /* Initialize Vertex, Edge */
-  igraphmodule_VertexType.tp_clear = (inquiry)igraphmodule_Vertex_clear;
-  if (PyType_Ready(&igraphmodule_VertexType) < 0)
-    INITERROR;
-  
-  igraphmodule_EdgeType.tp_clear = (inquiry)igraphmodule_Edge_clear;
-  if (PyType_Ready(&igraphmodule_EdgeType) < 0)
+  /* Run basic initialization of the pyhelpers.c module */
+  if (igraphmodule_helpers_init()) {
     INITERROR;
+  }
 
-  /* Initialize Graph, BFSIter, ARPACKOptions etc */
-  if (igraphmodule_ARPACKOptions_register_type())
-    INITERROR;
-  if (PyType_Ready(&igraphmodule_GraphType) < 0)
-    INITERROR;
-  if (PyType_Ready(&igraphmodule_BFSIterType) < 0)
-    INITERROR;
-  if (PyType_Ready(&igraphmodule_DFSIterType) < 0)
+  /* Initialize types */
+  if (
+    igraphmodule_ARPACKOptions_register_type() ||
+    igraphmodule_BFSIter_register_type() ||
+    igraphmodule_DFSIter_register_type() ||
+    igraphmodule_Edge_register_type() ||
+    igraphmodule_EdgeSeq_register_type() ||
+    igraphmodule_Graph_register_type() ||
+    igraphmodule_Vertex_register_type() ||
+    igraphmodule_VertexSeq_register_type()
+  ) {
     INITERROR;
+  }
 
   /* Initialize the core module */
   m = PyModule_Create(&moduledef);
@@ -840,14 +941,14 @@ PyObject* PyInit__igraph(void)
   igraphmodule_init_rng(m);
 
   /* Add the types to the core module */
-  PyModule_AddObject(m, "GraphBase", (PyObject*)&igraphmodule_GraphType);
-  PyModule_AddObject(m, "BFSIter", (PyObject*)&igraphmodule_BFSIterType);
-  PyModule_AddObject(m, "DFSIter", (PyObject*)&igraphmodule_DFSIterType);
+  PyModule_AddObject(m, "GraphBase", (PyObject*)igraphmodule_GraphType);
+  PyModule_AddObject(m, "BFSIter", (PyObject*)igraphmodule_BFSIterType);
+  PyModule_AddObject(m, "DFSIter", (PyObject*)igraphmodule_DFSIterType);
   PyModule_AddObject(m, "ARPACKOptions", (PyObject*)igraphmodule_ARPACKOptionsType);
-  PyModule_AddObject(m, "Edge", (PyObject*)&igraphmodule_EdgeType);
-  PyModule_AddObject(m, "EdgeSeq", (PyObject*)&igraphmodule_EdgeSeqType);
-  PyModule_AddObject(m, "Vertex", (PyObject*)&igraphmodule_VertexType);
-  PyModule_AddObject(m, "VertexSeq", (PyObject*)&igraphmodule_VertexSeqType);
+  PyModule_AddObject(m, "Edge", (PyObject*)igraphmodule_EdgeType);
+  PyModule_AddObject(m, "EdgeSeq", (PyObject*)igraphmodule_EdgeSeqType);
+  PyModule_AddObject(m, "Vertex", (PyObject*)igraphmodule_VertexType);
+  PyModule_AddObject(m, "VertexSeq", (PyObject*)igraphmodule_VertexSeqType);
  
   /* Internal error exception type */
   igraphmodule_InternalError =
@@ -855,7 +956,10 @@ PyObject* PyInit__igraph(void)
   PyModule_AddObject(m, "InternalError", igraphmodule_InternalError);
 
   /* ARPACK default options variable */
-  igraphmodule_arpack_options_default = igraphmodule_ARPACKOptions_new();
+  igraphmodule_arpack_options_default = PyObject_CallFunction((PyObject*) igraphmodule_ARPACKOptionsType, 0);
+  if (igraphmodule_arpack_options_default == NULL)
+    INITERROR;
+
   PyModule_AddObject(m, "arpack_options", igraphmodule_arpack_options_default);
 
   /* Useful constants */
@@ -904,6 +1008,8 @@ PyObject* PyInit__igraph(void)
   PyModule_AddIntConstant(m, "LOOPS_SW", IGRAPH_LOOPS_SW);
   PyModule_AddIntConstant(m, "MULTI_SW", IGRAPH_MULTI_SW);
 
+  PyModule_AddIntConstant(m, "INTEGER_SIZE", IGRAPH_INTEGER_SIZE);
+
   /* More useful constants */
   {
     const char* version;
