@@ -35,17 +35,16 @@
  * \defgroup python_interface_vertex Vertex object
  */
 
-PyTypeObject igraphmodule_VertexType;
+PyTypeObject* igraphmodule_VertexType;
+
+PyObject* igraphmodule_Vertex_attributes(igraphmodule_VertexObject* self);
 
 /**
  * \ingroup python_interface_vertex
  * \brief Checks whether the given Python object is a vertex
  */
 int igraphmodule_Vertex_Check(PyObject* obj) {
-  if (!obj)
-    return 0;
-  
-  return PyObject_IsInstance(obj, (PyObject*)(&igraphmodule_VertexType));
+  return obj ? PyObject_IsInstance(obj, (PyObject*)igraphmodule_VertexType) : 0;
 }
 
 /**
@@ -100,28 +99,35 @@ int igraphmodule_Vertex_Validate(PyObject* obj) {
  * (or they might even get invalidated).
  */
 PyObject* igraphmodule_Vertex_New(igraphmodule_GraphObject *gref, igraph_integer_t idx) {
-  igraphmodule_VertexObject* self;
-  self=PyObject_New(igraphmodule_VertexObject, &igraphmodule_VertexType);
-  if (self) {
-    RC_ALLOC("Vertex", self);
-    Py_INCREF(gref);
-    self->gref=gref;
-    self->idx=idx;
-    self->hash=-1;
-  }
-  return (PyObject*)self;
+  return PyObject_CallFunction((PyObject*) igraphmodule_VertexType, "On", gref, (Py_ssize_t) idx);
 }
 
 /**
  * \ingroup python_interface_vertex
- * \brief Clears the vertex's subobject (before deallocation)
+ * \brief Initialize a new vertex object for a given graph
+ * \return the initialized PyObject
  */
-int igraphmodule_Vertex_clear(igraphmodule_VertexObject *self) {
-  PyObject *tmp;
+static int igraphmodule_Vertex_init(igraphmodule_EdgeObject *self, PyObject *args, PyObject *kwds) {
+  static char *kwlist[] = { "graph", "vid", NULL };
+  PyObject *g, *index_o = Py_None;
+  igraph_integer_t vid;
+  igraph_t *graph;
 
-  tmp=(PyObject*)self->gref;
-  self->gref=NULL;
-  Py_XDECREF(tmp);
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O", kwlist,
+    igraphmodule_GraphType, &g, &index_o)) {
+    return -1;
+  }
+
+  graph = &((igraphmodule_GraphObject*)g)->g;
+
+  if (igraphmodule_PyObject_to_vid(index_o, &vid, graph)) {
+    return -1;
+  }
+
+  Py_INCREF(g);
+  self->gref = (igraphmodule_GraphObject*)g;
+  self->idx = vid;
+  self->hash = -1;
 
   return 0;
 }
@@ -130,12 +136,10 @@ int igraphmodule_Vertex_clear(igraphmodule_VertexObject *self) {
  * \ingroup python_interface_vertex
  * \brief Deallocates a Python representation of a given vertex object
  */
-void igraphmodule_Vertex_dealloc(igraphmodule_VertexObject* self) {
-  igraphmodule_Vertex_clear(self);
-
+static void igraphmodule_Vertex_dealloc(igraphmodule_VertexObject* self) {
   RC_DEALLOC("Vertex", self);
-
-  PyObject_Del((PyObject*)self);
+  Py_CLEAR(self->gref);
+  PY_FREE_AND_DECREF_TYPE(self, igraphmodule_VertexType);
 }
 
 /** \ingroup python_interface_vertex
@@ -143,16 +147,17 @@ void igraphmodule_Vertex_dealloc(igraphmodule_VertexObject* self) {
  * 
  * \return the formatted textual representation as a \c PyObject
  */
-PyObject* igraphmodule_Vertex_repr(igraphmodule_VertexObject *self) {
+static PyObject* igraphmodule_Vertex_repr(igraphmodule_VertexObject *self) {
   PyObject *s;
   PyObject *attrs;
 
   attrs = igraphmodule_Vertex_attributes(self);
-  if (attrs == 0)
+  if (attrs == 0) {
     return NULL;
+  }
 
-  s = PyUnicode_FromFormat("igraph.Vertex(%R, %ld, %R)",
-      (PyObject*)self->gref, (long int)self->idx, attrs);
+  s = PyUnicode_FromFormat("igraph.Vertex(%R, %" IGRAPH_PRId ", %R)",
+      (PyObject*)self->gref, self->idx, attrs);
   Py_DECREF(attrs);
 
   return s;
@@ -161,7 +166,7 @@ PyObject* igraphmodule_Vertex_repr(igraphmodule_VertexObject *self) {
 /** \ingroup python_interface_vertex
  * \brief Returns the hash code of the vertex
  */
-long igraphmodule_Vertex_hash(igraphmodule_VertexObject* self) {
+static long igraphmodule_Vertex_hash(igraphmodule_VertexObject* self) {
   long hash_graph;
   long hash_index;
   long result;
@@ -170,7 +175,7 @@ long igraphmodule_Vertex_hash(igraphmodule_VertexObject* self) {
   if (self->hash != -1)
     return self->hash;
 
-  index_o = PyLong_FromLong((long int)self->idx);
+  index_o = igraphmodule_integer_t_to_PyObject(self->idx);
   if (index_o == 0)
     return -1;
 
@@ -198,7 +203,7 @@ long igraphmodule_Vertex_hash(igraphmodule_VertexObject* self) {
 /** \ingroup python_interface_vertex
  * \brief Rich comparison of a vertex with another
  */
-PyObject* igraphmodule_Vertex_richcompare(igraphmodule_VertexObject *a,
+static PyObject* igraphmodule_Vertex_richcompare(igraphmodule_VertexObject *a,
     PyObject *b, int op) {
 
   igraphmodule_VertexObject* self = a;
@@ -242,17 +247,18 @@ PyObject* igraphmodule_Vertex_richcompare(igraphmodule_VertexObject *a,
 Py_ssize_t igraphmodule_Vertex_attribute_count(igraphmodule_VertexObject* self) {
   igraphmodule_GraphObject *o = self->gref;
   
-  if (!o) return 0;
-  if (!((PyObject**)o->g.attr)[1]) return 0;
-  return PyDict_Size(((PyObject**)o->g.attr)[1]);
+  if (!o || !((PyObject**)o->g.attr)[1]) {
+    return 0;
+  } else {
+    return PyDict_Size(((PyObject**)o->g.attr)[1]);
+  }
 }
 
 /** \ingroup python_interface_vertex
  * \brief Returns the list of attribute names
  */
 PyObject* igraphmodule_Vertex_attribute_names(igraphmodule_VertexObject* self) {
-  if (!self->gref) return NULL;
-  return igraphmodule_Graph_vertex_attributes(self->gref);
+  return self->gref ? igraphmodule_Graph_vertex_attributes(self->gref) : 0;
 }
 
 /** \ingroup python_interface_vertex
@@ -261,22 +267,25 @@ PyObject* igraphmodule_Vertex_attribute_names(igraphmodule_VertexObject* self) {
 PyObject* igraphmodule_Vertex_attributes(igraphmodule_VertexObject* self) {
   igraphmodule_GraphObject *o = self->gref;
   PyObject *names, *dict;
-  long i, n;
+  Py_ssize_t i, n;
 
-  if (!igraphmodule_Vertex_Validate((PyObject*)self))
+  if (!igraphmodule_Vertex_Validate((PyObject*)self)) {
     return 0;
+  }
 
-  dict=PyDict_New();
-  if (!dict) return NULL;
+  dict = PyDict_New();
+  if (!dict) {
+    return NULL;
+  }
 
-  names=igraphmodule_Graph_vertex_attributes(o);
+  names = igraphmodule_Graph_vertex_attributes(o);
   if (!names) {
     Py_DECREF(dict);
     return NULL;
   }
 
-  n=PyList_Size(names);
-  for (i=0; i<n; i++) {
+  n = PyList_Size(names);
+  for (i = 0; i < n; i++) {
     PyObject *name = PyList_GetItem(names, i);
     if (name) {
       PyObject *dictit;
@@ -287,7 +296,15 @@ PyObject* igraphmodule_Vertex_attributes(igraphmodule_VertexObject* self) {
           /* No need to Py_INCREF, PyDict_SetItem will do that */
           PyDict_SetItem(dict, name, value);
         }
+      } else {
+        Py_DECREF(dict);
+        Py_DECREF(names);
+        return NULL;
       }
+    } else {
+      Py_DECREF(dict);
+      Py_DECREF(names);
+      return NULL;
     }
   }
 
@@ -514,24 +531,24 @@ int igraphmodule_Vertex_set_attribute(igraphmodule_VertexObject* self, PyObject*
   /* result is NULL, check whether there was an error */
   if (!PyErr_Occurred()) {
     /* no, there wasn't, so we must simply add the attribute */
-    int n=(int)igraph_vcount(&o->g), i;
-    result=PyList_New(n);
-    for (i=0; i<n; i++) {
+    igraph_integer_t i, n = igraph_vcount(&o->g);
+    result = PyList_New(n);
+    for (i = 0; i < n; i++) {
       if (i != self->idx) {
-    Py_INCREF(Py_None);
-    if (PyList_SetItem(result, i, Py_None) == -1) {
-      Py_DECREF(Py_None);
-      Py_DECREF(result);
-      return -1;
-    }
+        Py_INCREF(Py_None);
+        if (PyList_SetItem(result, i, Py_None) == -1) {
+          Py_DECREF(Py_None);
+          Py_DECREF(result);
+          return -1;
+        }
       } else {
-    /* Same game with the reference count here */
-    Py_INCREF(v);
-    if (PyList_SetItem(result, i, v) == -1) {
-      Py_DECREF(v);
-      Py_DECREF(result);
-      return -1;
-    }
+        /* Same game with the reference count here */
+        Py_INCREF(v);
+        if (PyList_SetItem(result, i, v) == -1) {
+          Py_DECREF(v);
+          Py_DECREF(result);
+          return -1;
+        }
       }
     }
     if (PyDict_SetItem(((PyObject**)o->g.attr)[1], k, result) == -1) {
@@ -550,7 +567,7 @@ int igraphmodule_Vertex_set_attribute(igraphmodule_VertexObject* self, PyObject*
  * Returns the vertex index
  */
 PyObject* igraphmodule_Vertex_get_index(igraphmodule_VertexObject* self, void* closure) {
-  return PyLong_FromLong((long int)self->idx);
+  return igraphmodule_integer_t_to_PyObject(self->idx);
 }
 
 /**
@@ -561,14 +578,6 @@ igraph_integer_t igraphmodule_Vertex_get_index_igraph_integer(igraphmodule_Verte
   return self->idx;
 }
 
-/**
- * \ingroup python_interface_vertex
- * Returns the vertex index as an ordinary C long
- */
-long igraphmodule_Vertex_get_index_long(igraphmodule_VertexObject* self) {
-  return (long)self->idx;
-}
-
 /**
  * \ingroup python_interface_vertexseq
  * Returns the graph where the vertex belongs
@@ -607,23 +616,36 @@ static PyObject* _convert_to_edge_list(igraphmodule_VertexObject* vertex, PyObje
 
   n = PyList_Size(obj);
   for (i = 0; i < n; i++) {
-    PyObject* idx = PyList_GET_ITEM(obj, i);
-    PyObject* v;
-    int idx_int;
+    PyObject* idx = PyList_GetItem(obj, i);
+    PyObject* edge;
+    igraph_integer_t idx_int;
 
+    if (!idx) {
+      return NULL;
+    }
+  
     if (!PyLong_Check(idx)) {
       PyErr_SetString(PyExc_TypeError, "_convert_to_edge_list expected list of integers");
       return NULL;
     }
 
-    if (PyLong_AsInt(idx, &idx_int))
+    if (igraphmodule_PyObject_to_integer_t(idx, &idx_int)) {
       return NULL;
+    }
 
-    v = igraphmodule_Edge_New(vertex->gref, idx_int);
-    PyList_SetItem(obj, i, v);   /* reference to v stolen, reference to idx discarded */
+    edge = igraphmodule_Edge_New(vertex->gref, idx_int);
+    if (!edge) {
+      return NULL;
+    }
+
+    if (PyList_SetItem(obj, i, edge)) {  /* reference to v stolen, reference to idx discarded */
+      Py_DECREF(edge);
+      return NULL;
+    }
   }
 
   Py_INCREF(obj);
+
   return obj;
 }
 
@@ -639,20 +661,32 @@ static PyObject* _convert_to_vertex_list(igraphmodule_VertexObject* vertex, PyOb
 
   n = PyList_Size(obj);
   for (i = 0; i < n; i++) {
-    PyObject* idx = PyList_GET_ITEM(obj, i);
+    PyObject* idx = PyList_GetItem(obj, i);
     PyObject* v;
-    int idx_int;
+    igraph_integer_t idx_int;
 
+    if (!idx) {
+      return NULL;
+    }
+  
     if (!PyLong_Check(idx)) {
       PyErr_SetString(PyExc_TypeError, "_convert_to_vertex_list expected list of integers");
       return NULL;
     }
 
-    if (PyLong_AsInt(idx, &idx_int))
+    if (igraphmodule_PyObject_to_integer_t(idx, &idx_int)) {
       return NULL;
+    }
 
     v = igraphmodule_Vertex_New(vertex->gref, idx_int);
-    PyList_SetItem(obj, i, v);   /* reference to v stolen, reference to idx discarded */
+    if (!v) {
+      return NULL;
+    }
+
+    if (PyList_SetItem(obj, i, v)) {  /* reference to v stolen, reference to idx discarded */
+      Py_DECREF(v);
+      return NULL;
+    }
   }
 
   Py_INCREF(obj);
@@ -662,18 +696,25 @@ static PyObject* _convert_to_vertex_list(igraphmodule_VertexObject* vertex, PyOb
 #define GRAPH_PROXY_METHOD_PP(FUNC, METHODNAME, POSTPROCESS) \
     PyObject* igraphmodule_Vertex_##FUNC(igraphmodule_VertexObject* self, PyObject* args, PyObject* kwds) { \
       PyObject *new_args, *item, *result;                     \
-      long int i, num_args = args ? PyTuple_Size(args)+1 : 1; \
+      Py_ssize_t i, num_args = args ? PyTuple_Size(args) + 1 : 1; \
                                                               \
       /* Prepend ourselves to args */                         \
       new_args = PyTuple_New(num_args);                       \
-      Py_INCREF(self); PyTuple_SET_ITEM(new_args, 0, (PyObject*)self);   \
+      Py_INCREF(self);                                        \
+      PyTuple_SetItem(new_args, 0, (PyObject*)self);          \
       for (i = 1; i < num_args; i++) {                        \
-        item = PyTuple_GET_ITEM(args, i-1);                   \
-        Py_INCREF(item); PyTuple_SET_ITEM(new_args, i, item); \
+        item = PyTuple_GetItem(args, i - 1);                  \
+        Py_INCREF(item);                                      \
+        PyTuple_SetItem(new_args, i, item);                   \
       }                                                       \
                                                               \
       /* Get the method instance */                           \
       item = PyObject_GetAttrString((PyObject*)(self->gref), METHODNAME);  \
+      if (item == 0) {                                        \
+        Py_DECREF(new_args);                                  \
+        return 0;                                             \
+      }                                                       \
+                                                              \
       result = PyObject_Call(item, new_args, kwds);           \
       Py_DECREF(item);                                        \
       Py_DECREF(new_args);                                    \
@@ -684,6 +725,7 @@ static PyObject* _convert_to_vertex_list(igraphmodule_VertexObject* vertex, PyOb
         Py_DECREF(result);                                    \
         return pp_result;                                     \
       }                                                       \
+                                                              \
       return NULL;                                            \
     }
 
@@ -696,6 +738,7 @@ GRAPH_PROXY_METHOD(constraint, "constraint");
 GRAPH_PROXY_METHOD(degree, "degree");
 GRAPH_PROXY_METHOD(delete, "delete_vertices");
 GRAPH_PROXY_METHOD(diversity, "diversity");
+GRAPH_PROXY_METHOD(distances, "distances");
 GRAPH_PROXY_METHOD(eccentricity, "eccentricity");
 GRAPH_PROXY_METHOD(get_shortest_paths, "get_shortest_paths");
 GRAPH_PROXY_METHOD_PP(incident, "incident", _convert_to_edge_list);
@@ -715,16 +758,25 @@ GRAPH_PROXY_METHOD_PP(successors, "successors", _convert_to_vertex_list);
 
 #define GRAPH_PROXY_METHOD_SPEC(FUNC, METHODNAME) \
   {METHODNAME, (PyCFunction)igraphmodule_Vertex_##FUNC, METH_VARARGS | METH_KEYWORDS, \
+    METHODNAME "(*args, **kwds)\n--\n\n" \
     "Proxy method to L{Graph." METHODNAME "()<igraph._igraph.GraphBase." METHODNAME ">}\n\n"              \
     "This method calls the C{" METHODNAME "()} method of the L{Graph} class " \
     "with this vertex as the first argument, and returns the result.\n\n"\
     "@see: L{Graph." METHODNAME "()<igraph._igraph.GraphBase." METHODNAME ">} for details."}
 #define GRAPH_PROXY_METHOD_SPEC_2(FUNC, METHODNAME, METHODNAME_IN_GRAPH) \
   {METHODNAME, (PyCFunction)igraphmodule_Vertex_##FUNC, METH_VARARGS | METH_KEYWORDS, \
+    METHODNAME "(*args, **kwds)\n--\n\n" \
     "Proxy method to L{Graph." METHODNAME_IN_GRAPH "()<igraph._igraph.GraphBase." METHODNAME_IN_GRAPH ">}\n\n"              \
     "This method calls the C{" METHODNAME_IN_GRAPH "} method of the L{Graph} class " \
     "with this vertex as the first argument, and returns the result.\n\n"\
     "@see: L{Graph." METHODNAME_IN_GRAPH "()<igraph._igraph.GraphBase." METHODNAME_IN_GRAPH ">} for details."}
+#define GRAPH_PROXY_METHOD_SPEC_3(FUNC, METHODNAME) \
+  {METHODNAME, (PyCFunction)igraphmodule_Vertex_##FUNC, METH_VARARGS | METH_KEYWORDS, \
+    METHODNAME "(*args, **kwds)\n--\n\n" \
+    "Proxy method to L{Graph." METHODNAME "()}\n\n"              \
+    "This method calls the C{" METHODNAME "()} method of the L{Graph} class " \
+    "with this vertex as the first argument, and returns the result.\n\n"\
+    "@see: L{Graph." METHODNAME "()} for details."}
 
 /**
  * \ingroup python_interface_vertex
@@ -752,18 +804,21 @@ PyMethodDef igraphmodule_Vertex_methods[] = {
     "dictionaries."
   },
   {"all_edges", (PyCFunction)igraphmodule_Vertex_all_edges, METH_NOARGS,
+    "all_edges()\n--\n\n"
     "Proxy method to L{Graph.incident(..., mode=\"all\")<igraph._igraph.GraphBase.incident()>}\n\n" \
     "This method calls the incident() method of the L{Graph} class " \
     "with this vertex as the first argument and \"all\" as the mode " \
     "argument, and returns the result.\n\n"\
     "@see: L{Graph.incident()<igraph._igraph.GraphBase.incident()>} for details."},
   {"in_edges", (PyCFunction)igraphmodule_Vertex_in_edges, METH_NOARGS,
+    "in_edges()\n--\n\n"
     "Proxy method to L{Graph.incident(..., mode=\"in\")<igraph._igraph.GraphBase.incident()>}\n\n" \
     "This method calls the incident() method of the L{Graph} class " \
     "with this vertex as the first argument and \"in\" as the mode " \
     "argument, and returns the result.\n\n"\
     "@see: L{Graph.incident()<igraph._igraph.GraphBase.incident()>} for details."},
   {"out_edges", (PyCFunction)igraphmodule_Vertex_out_edges, METH_NOARGS,
+    "out_edges()\n--\n\n"
     "Proxy method to L{Graph.incident(..., mode=\"out\")<igraph._igraph.GraphBase.incident()>}\n\n" \
     "This method calls the incident() method of the L{Graph} class " \
     "with this vertex as the first argument and \"out\" as the mode " \
@@ -778,12 +833,12 @@ PyMethodDef igraphmodule_Vertex_methods[] = {
   GRAPH_PROXY_METHOD_SPEC(eccentricity, "eccentricity"),
   GRAPH_PROXY_METHOD_SPEC(get_shortest_paths, "get_shortest_paths"),
   GRAPH_PROXY_METHOD_SPEC(incident, "incident"),
-  GRAPH_PROXY_METHOD_SPEC(indegree, "indegree"),
+  GRAPH_PROXY_METHOD_SPEC_3(indegree, "indegree"),
   GRAPH_PROXY_METHOD_SPEC(is_minimal_separator, "is_minimal_separator"),
   GRAPH_PROXY_METHOD_SPEC(is_separator, "is_separator"),
   GRAPH_PROXY_METHOD_SPEC(neighbors, "neighbors"),
-  GRAPH_PROXY_METHOD_SPEC(outdegree, "outdegree"),
-  GRAPH_PROXY_METHOD_SPEC(pagerank, "pagerank"),
+  GRAPH_PROXY_METHOD_SPEC_3(outdegree, "outdegree"),
+  GRAPH_PROXY_METHOD_SPEC_3(pagerank, "pagerank"),
   GRAPH_PROXY_METHOD_SPEC(predecessors, "predecessors"),
   GRAPH_PROXY_METHOD_SPEC(personalized_pagerank, "personalized_pagerank"),
   GRAPH_PROXY_METHOD_SPEC(shortest_paths, "shortest_paths"),
@@ -795,20 +850,6 @@ PyMethodDef igraphmodule_Vertex_methods[] = {
 #undef GRAPH_PROXY_METHOD_SPEC
 #undef GRAPH_PROXY_METHOD_SPEC_2
 
-/** \ingroup python_interface_vertex
- * This structure is the collection of functions necessary to implement
- * the vertex as a mapping (i.e. to allow the retrieval and setting of
- * igraph attributes in Python as if it were of a Python mapping type)
- */
-PyMappingMethods igraphmodule_Vertex_as_mapping = {
-  // returns the number of vertex attributes
-  (lenfunc)igraphmodule_Vertex_attribute_count,
-  // returns an attribute by name
-  (binaryfunc)igraphmodule_Vertex_get_attribute,
-  // sets an attribute by name
-  (objobjargproc)igraphmodule_Vertex_set_attribute
-};
-
 /**
  * \ingroup python_interface_vertex
  * Getter/setter table for the \c igraph.Vertex object
@@ -823,32 +864,8 @@ PyGetSetDef igraphmodule_Vertex_getseters[] = {
   {NULL}
 };
 
-/** \ingroup python_interface_vertex
- * Python type object referencing the methods Python calls when it performs various operations on
- * a vertex of a graph
- */
-PyTypeObject igraphmodule_VertexType =
-{
-  PyVarObject_HEAD_INIT(0, 0)
-  "igraph.Vertex",                            /* tp_name */
-  sizeof(igraphmodule_VertexObject),          /* tp_basicsize */
-  0,                                          /* tp_itemsize */
-  (destructor)igraphmodule_Vertex_dealloc,    /* tp_dealloc */
-  0,                                          /* tp_print */
-  0,                                          /* tp_getattr */
-  0,                                          /* tp_setattr */
-  0,                                          /* tp_compare (2.x) / tp_reserved (3.x) */
-  (reprfunc)igraphmodule_Vertex_repr,         /* tp_repr */
-  0,                                          /* tp_as_number */
-  0,                                          /* tp_as_sequence */
-  &igraphmodule_Vertex_as_mapping,            /* tp_as_mapping */
-  (hashfunc)igraphmodule_Vertex_hash,         /* tp_hash */
-  0,                                          /* tp_call */
-  0,                                          /* tp_str */
-  0,                                          /* tp_getattro */
-  0,                                          /* tp_setattro */
-  0,                                          /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,   /* tp_flags */
+PyDoc_STRVAR(
+  igraphmodule_Vertex_doc,
   "Class representing a single vertex in a graph.\n\n"
   "The vertex is referenced by its index, so if the underlying graph\n"
   "changes, the semantics of the vertex object might change as well\n"
@@ -857,15 +874,35 @@ PyTypeObject igraphmodule_VertexType =
   "as a hash:\n\n"
   "  >>> v[\"color\"] = \"red\"                  #doctest: +SKIP\n"
   "  >>> print(v[\"color\"])                     #doctest: +SKIP\n"
-  "  red\n", /* tp_doc */
-  0,                                          /* tp_traverse */
-  0,                                          /* tp_clear */
-  (richcmpfunc)igraphmodule_Vertex_richcompare, /* tp_richcompare */
-  0,                                          /* tp_weaklistoffset */
-  0,                                          /* tp_iter */
-  0,                                          /* tp_iternext */
-  igraphmodule_Vertex_methods,                /* tp_methods */
-  0,                                          /* tp_members */
-  igraphmodule_Vertex_getseters,              /* tp_getset */
-};
+  "  red\n"
+);
 
+int igraphmodule_Vertex_register_type() {
+  PyType_Slot slots[] = {
+    { Py_tp_init, igraphmodule_Vertex_init },
+    { Py_tp_dealloc, igraphmodule_Vertex_dealloc },
+    { Py_tp_hash, igraphmodule_Vertex_hash },
+    { Py_tp_repr, igraphmodule_Vertex_repr },
+    { Py_tp_richcompare, igraphmodule_Vertex_richcompare },
+    { Py_tp_methods, igraphmodule_Vertex_methods },
+    { Py_tp_getset, igraphmodule_Vertex_getseters },
+    { Py_tp_doc, (void*) igraphmodule_Vertex_doc },
+
+    { Py_mp_length, igraphmodule_Vertex_attribute_count },
+    { Py_mp_subscript, igraphmodule_Vertex_get_attribute },
+    { Py_mp_ass_subscript, igraphmodule_Vertex_set_attribute },
+  
+    { 0 }
+  };
+
+  PyType_Spec spec = {
+    "igraph.Vertex",                            /* name */
+    sizeof(igraphmodule_VertexObject),          /* basicsize */
+    0,                                          /* itemsize */
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,   /* flags */
+    slots,                                      /* slots */
+  };
+
+  igraphmodule_VertexType = (PyTypeObject*) PyType_FromSpec(&spec);
+  return igraphmodule_VertexType == 0;
+}
