@@ -330,20 +330,17 @@ class ForeignTests(unittest.TestCase):
         ) as tmpfname:
             g = Graph.Read_Adjacency(tmpfname)
             self.assertTrue(isinstance(g, Graph))
-            self.assertTrue(
-                g.vcount() == 6
-                and g.ecount() == 18
-                and g.is_directed()
-                and "weight" not in g.edge_attributes()
-            )
+            self.assertEqual(g.vcount(), 6)
+            self.assertEqual(g.ecount(), 18)
+            self.assertTrue(g.is_directed())
+            self.assertTrue("weight" not in g.edge_attributes())
+
             g = Graph.Read_Adjacency(tmpfname, attribute="weight")
             self.assertTrue(isinstance(g, Graph))
-            self.assertTrue(
-                g.vcount() == 6
-                and g.ecount() == 12
-                and g.is_directed()
-                and g.es["weight"] == [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2]
-            )
+            self.assertEqual(g.vcount(), 6)
+            self.assertEqual(g.ecount(), 12)
+            self.assertTrue(g.is_directed())
+            self.assertTrue(g.es["weight"] == [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2])
 
             g.write_adjacency(tmpfname)
 
@@ -428,6 +425,148 @@ class ForeignTests(unittest.TestCase):
             self.assertTrue(g.vcount() == 3 and g.ecount() == 1 and not g.is_directed())
             g.write_pickle(tmpfname)
 
+    def testDictList(self):
+        g = Graph.Full(3)
+
+        # Check with vertex ids
+        self.assertEqual(
+            g.to_dict_list(),
+            (
+                [{}, {}, {}],
+                [
+                    {"source": 0, "target": 1},
+                    {"source": 0, "target": 2},
+                    {"source": 1, "target": 2},
+                ],
+            ),
+        )
+
+        # Check failure for vertex names
+        self.assertRaises(AttributeError, g.to_dict_list, False)
+
+        # Check with vertex names
+        g.vs["name"] = ["apple", "pear", "peach"]
+        self.assertEqual(
+            g.to_dict_list(),
+            (
+                [{"name": "apple"}, {"name": "pear"}, {"name": "peach"}],
+                [
+                    {"source": 0, "target": 1},
+                    {"source": 0, "target": 2},
+                    {"source": 1, "target": 2},
+                ],
+            ),
+        )
+        self.assertEqual(
+            g.to_dict_list(use_vids=False),
+            (
+                [{"name": "apple"}, {"name": "pear"}, {"name": "peach"}],
+                [
+                    {"source": "apple", "target": "pear"},
+                    {"source": "apple", "target": "peach"},
+                    {"source": "pear", "target": "peach"},
+                ],
+            ),
+        )
+
+    def testTupleList(self):
+        g = Graph.Full(3)
+
+        # Check with vertex ids
+        self.assertEqual(
+            g.to_tuple_list(),
+            [(0, 1), (0, 2), (1, 2)],
+        )
+
+        # Check failure for edge names
+        self.assertRaises(AttributeError, g.to_tuple_list, True, "name")
+
+        # Edge attributes
+        g.es["name"] = ["first_edge", "second", None]
+        self.assertEqual(
+            g.to_tuple_list(edge_attrs="name"),
+            [(0, 1, "first_edge"), (0, 2, "second"), (1, 2, None)],
+        )
+        self.assertEqual(
+            g.to_tuple_list(edge_attrs=["name"]),
+            [(0, 1, "first_edge"), (0, 2, "second"), (1, 2, None)],
+        )
+
+        # Missing vertex names
+        self.assertRaises(AttributeError, g.to_tuple_list, False)
+
+        # Vertex names
+        g.vs["name"] = ["apple", "pear", "peach"]
+        self.assertEqual(
+            g.to_tuple_list(use_vids=False, edge_attrs="name"),
+            [
+                ("apple", "pear", "first_edge"),
+                ("apple", "peach", "second"),
+                ("pear", "peach", None),
+            ],
+        )
+
+    def testSequenceDict(self):
+        g = Graph.Full(3)
+
+        # Check with vertex ids
+        self.assertEqual(g.to_list_dict(), {0: [1, 2], 1: [2]})
+        self.assertEqual(
+            g.to_list_dict(sequence_constructor=tuple),
+            {0: (1, 2), 1: (2,)},
+        )
+
+        # Check failure for vertex names
+        self.assertRaises(AttributeError, g.to_list_dict, False)
+
+        # Check with vertex names
+        g.vs["name"] = ["apple", "pear", "peach"]
+        self.assertEqual(
+            g.to_list_dict(use_vids=False),
+            {"apple": ["pear", "peach"], "pear": ["peach"]},
+        )
+
+    def testDictDict(self):
+        g = Graph([(0, 1), (0, 2), (1, 2)])
+
+        # Check with vertex ids, no edge attrs
+        self.assertEqual(
+            g.to_dict_dict(),
+            {0: {1: {}, 2: {}}, 1: {2: {}}},
+        )
+
+        # With vertex ids, edge attrs
+        g.es["name"] = ["first_edge", "second", None]
+        # Check with vertex ids, incomplete edge attrs
+        self.assertEqual(
+            g.to_dict_dict(),
+            {
+                0: {1: {"name": "first_edge"}, 2: {"name": "second"}},
+                1: {2: {"name": None}},
+            },
+        )
+        self.assertEqual(
+            g.to_dict_dict(skip_none=True),
+            {0: {1: {"name": "first_edge"}, 2: {"name": "second"}}, 1: {2: {}}},
+        )
+
+        # With vertex names
+        g.vs["name"] = ["apple", "pear", "peach"]
+        self.assertEqual(
+            g.to_dict_dict(use_vids=False),
+            {
+                "apple": {"pear": {"name": "first_edge"}, "peach": {"name": "second"}},
+                "pear": {"peach": {"name": None}},
+            },
+        )
+        self.assertEqual(
+            g.to_dict_dict(use_vids=False, skip_none=True),
+            {
+                "apple": {"pear": {"name": "first_edge"}, "peach": {"name": "second"}},
+                "pear": {"peach": {}},
+            },
+        )
+
     @unittest.skipIf(pd is None, "test case depends on Pandas")
     def testVertexDataFrames(self):
         g = Graph([(0, 1), (0, 2), (0, 3), (1, 2), (2, 4)])
@@ -506,7 +645,6 @@ class ForeignTests(unittest.TestCase):
         i = 2 + list(df.columns[2:]).index("source")
         self.assertEqual(list(df.iloc[:, i]), g.es["source"])
 
-
     @unittest.skipIf(nx is None, "test case depends on networkx")
     def testGraphNetworkx(self):
         # Undirected
@@ -552,6 +690,19 @@ class ForeignTests(unittest.TestCase):
         self.assertEqual(g.vcount(), g2.vcount())
         self.assertEqual(sorted(g.get_edgelist()), sorted(g2.get_edgelist()))
 
+        # Test networkx with custom node hashables
+        # In this case each node is a tuple of ints
+        g_nx = nx.DiGraph()
+        inf = float("inf")
+        g_nx.add_edges_from((
+            ((0, 0), (1, 1), {"weight": 3.0}),
+            ((0, 0), (1, 0), {"weight": inf}),
+        ))
+        g = Graph.from_networkx(g_nx)
+        self.assertTrue(g.is_directed())
+        self.assertEqual(g.vcount(), 3)
+        self.assertTrue(g.es["weight"] == [3.0, inf] or g.es["weight"] == [inf, 3.0])
+
     @unittest.skipIf(nx is None, "test case depends on networkx")
     def testMultigraphNetworkx(self):
         # Undirected
@@ -572,8 +723,15 @@ class ForeignTests(unittest.TestCase):
 
         # Test attributes
         self.assertEqual(g.attributes(), g2.attributes())
-        self.assertEqual(sorted(["vattr", "_nx_name"]), sorted(g2.vertex_attributes()))
-        self.assertEqual(g.edge_attributes(), g2.edge_attributes())
+        self.assertEqual(
+            sorted(["vattr", "_nx_name"]),
+            sorted(g2.vertex_attributes()),
+        )
+        self.assertEqual(
+            sorted(["eattr", "_nx_multiedge_key"]),
+            sorted(g2.edge_attributes()),
+        )
+
         # Testing parallel edges is a bit more tricky
         edge2_found = set()
         for edge in g.es:
@@ -726,7 +884,7 @@ class ForeignTests(unittest.TestCase):
 
 
 def suite():
-    foreign_suite = unittest.makeSuite(ForeignTests)
+    foreign_suite = unittest.defaultTestLoader.loadTestsFromTestCase(ForeignTests)
     return unittest.TestSuite([foreign_suite])
 
 
