@@ -127,42 +127,58 @@ class GeneratorTests(unittest.TestCase):
         self.assertTrue(g.is_directed())
         self.assertTrue(el == [(x, y) for x in range(1, 20) for y in range(x)])
 
-        self.assertRaises(InternalError, Graph.Full_Citation, -2)
+        self.assertRaises(ValueError, Graph.Full_Citation, -2)
 
     def testLCF(self):
         g1 = Graph.LCF(12, (5, -5), 6)
         g2 = Graph.Famous("Franklin")
         self.assertTrue(g1.isomorphic(g2))
-        self.assertRaises(InternalError, Graph.LCF, 12, (5, -5), -3)
+        self.assertRaises(ValueError, Graph.LCF, 12, (5, -5), -3)
 
     def testRealizeDegreeSequence(self):
         # Test case insensitivity of options too
         g = Graph.Realize_Degree_Sequence(
-            [1, 1], None, "simPLE", "smallest",
+            [1, 1],
+            None,
+            "simPLE",
+            "smallest",
         )
         self.assertFalse(g.is_directed())
         self.assertTrue(g.degree() == [1, 1])
 
         # Not implemented, should fail
         self.assertRaises(
-            NotImplementedError, Graph.Realize_Degree_Sequence,
-            [1, 1], None, "loops", "largest"
+            NotImplementedError,
+            Graph.Realize_Degree_Sequence,
+            [1, 1],
+            None,
+            "loops",
+            "largest",
         )
 
         g = Graph.Realize_Degree_Sequence(
-            [1, 1], None, "all", "largest",
+            [1, 1],
+            None,
+            "all",
+            "largest",
         )
         self.assertFalse(g.is_directed())
         self.assertTrue(g.degree() == [1, 1])
 
         g = Graph.Realize_Degree_Sequence(
-            [1, 1], None, "multi", "index",
+            [1, 1],
+            None,
+            "multi",
+            "index",
         )
         self.assertFalse(g.is_directed())
         self.assertTrue(g.degree() == [1, 1])
 
         g = Graph.Realize_Degree_Sequence(
-            [1, 1], [1, 1], "simple", "largest",
+            [1, 1],
+            [1, 1],
+            "simple",
+            "largest",
         )
         self.assertTrue(g.is_directed())
         self.assertTrue(g.indegree() == [1, 1])
@@ -170,17 +186,29 @@ class GeneratorTests(unittest.TestCase):
 
         # Not implemented, should fail
         self.assertRaises(
-            NotImplementedError, Graph.Realize_Degree_Sequence,
-            [1, 1], [1, 1], "multi", "largest"
+            NotImplementedError,
+            Graph.Realize_Degree_Sequence,
+            [1, 1],
+            [1, 1],
+            "multi",
+            "largest",
         )
 
         self.assertRaises(
-            ValueError, Graph.Realize_Degree_Sequence,
-            [1, 1], [1, 1], "should_fail", "index"
+            ValueError,
+            Graph.Realize_Degree_Sequence,
+            [1, 1],
+            [1, 1],
+            "should_fail",
+            "index",
         )
         self.assertRaises(
-            ValueError, Graph.Realize_Degree_Sequence,
-            [1, 1], [1, 1], "multi", "should_fail"
+            ValueError,
+            Graph.Realize_Degree_Sequence,
+            [1, 1],
+            [1, 1],
+            "multi",
+            "should_fail",
         )
 
         # Degree sequence of Zachary karate club, using optional arguments
@@ -204,6 +232,90 @@ class GeneratorTests(unittest.TestCase):
         # This is not a proper test, but should spot most errors
         self.assertTrue(g.is_directed() and deg_in == [2] * 8 and deg_out == [2] * 8)
 
+    def testLattice(self):
+        g = Graph.Lattice([4, 3], circular=False)
+        self.assertEqual(
+            sorted(sorted(x) for x in g.get_edgelist()),
+            [
+                [0, 1],
+                [0, 4],
+                [1, 2],
+                [1, 5],
+                [2, 3],
+                [2, 6],
+                [3, 7],
+                [4, 5],
+                [4, 8],
+                [5, 6],
+                [5, 9],
+                [6, 7],
+                [6, 10],
+                [7, 11],
+                [8, 9],
+                [9, 10],
+                [10, 11],
+            ],
+        )
+
+        g = Graph.Lattice([4, 3], circular=True)
+        self.assertEqual(
+            sorted(sorted(x) for x in g.get_edgelist()),
+            [
+                [0, 1],
+                [0, 3],
+                [0, 4],
+                [0, 8],
+                [1, 2],
+                [1, 5],
+                [1, 9],
+                [2, 3],
+                [2, 6],
+                [2, 10],
+                [3, 7],
+                [3, 11],
+                [4, 5],
+                [4, 7],
+                [4, 8],
+                [5, 6],
+                [5, 9],
+                [6, 7],
+                [6, 10],
+                [7, 11],
+                [8, 9],
+                [8, 11],
+                [9, 10],
+                [10, 11],
+            ],
+        )
+
+        g = Graph.Lattice([4, 3], circular=(False, 1))
+        self.assertEqual(
+            sorted(sorted(x) for x in g.get_edgelist()),
+            [
+                [0, 1],
+                [0, 4],
+                [0, 8],
+                [1, 2],
+                [1, 5],
+                [1, 9],
+                [2, 3],
+                [2, 6],
+                [2, 10],
+                [3, 7],
+                [3, 11],
+                [4, 5],
+                [4, 8],
+                [5, 6],
+                [5, 9],
+                [6, 7],
+                [6, 10],
+                [7, 11],
+                [8, 9],
+                [9, 10],
+                [10, 11],
+            ],
+        )
+
     def testSBM(self):
         pref_matrix = [[0.5, 0, 0], [0, 0, 0.5], [0, 0.5, 0]]
         n = 60
@@ -213,7 +325,7 @@ class GeneratorTests(unittest.TestCase):
         # Simple smoke tests for the expected structure of the graph
         self.assertTrue(g.is_simple())
         self.assertFalse(g.is_directed())
-        self.assertEqual([0] * 20 + [1] * 40, g.clusters().membership)
+        self.assertEqual([0] * 20 + [1] * 40, g.connected_components().membership)
         g2 = g.subgraph(list(range(20, 60)))
         self.assertTrue(not any(e.source // 20 == e.target // 20 for e in g2.es))
 
@@ -229,7 +341,7 @@ class GeneratorTests(unittest.TestCase):
         self.assertTrue(sum(g.is_loop()) == 0)
 
         # Check error conditions
-        self.assertRaises(InternalError, Graph.SBM, -1, pref_matrix, types)
+        self.assertRaises(ValueError, Graph.SBM, -1, pref_matrix, types)
         self.assertRaises(InternalError, Graph.SBM, 61, pref_matrix, types)
         pref_matrix[0][1] = 0.7
         self.assertRaises(InternalError, Graph.SBM, 60, pref_matrix, types)
@@ -364,13 +476,13 @@ class GeneratorTests(unittest.TestCase):
         edges = pd.DataFrame(
             [["C", "A", 0.4], ["A", "B", 0.1]], columns=[0, 1, "weight"]
         )
-        g = Graph.DataFrame(edges, directed=False)
+        g = Graph.DataFrame(edges, directed=False, use_vids=False)
         self.assertTrue(g.es["weight"] == [0.4, 0.1])
 
         vertices = pd.DataFrame(
             [["A", "blue"], ["B", "yellow"], ["C", "blue"]], columns=[0, "color"]
         )
-        g = Graph.DataFrame(edges, directed=True, vertices=vertices)
+        g = Graph.DataFrame(edges, directed=True, vertices=vertices, use_vids=False)
         self.assertTrue(g.vs["name"] == ["A", "B", "C"])
         self.assertTrue(g.vs["color"] == ["blue", "yellow", "blue"])
         self.assertTrue(g.es["weight"] == [0.4, 0.1])
@@ -380,21 +492,18 @@ class GeneratorTests(unittest.TestCase):
         vertices = pd.DataFrame(
             {"node": [1, 2, 3, 4, 5, 6], "label": ["1", "2", "3", "4", "5", "6"]}
         )[["node", "label"]]
-        g = Graph.DataFrame(
-            edges,
-            directed=True,
-            vertices=vertices
-        )
+        g = Graph.DataFrame(edges, directed=True, vertices=vertices, use_vids=False)
         self.assertTrue(g.vs["name"] == [1, 2, 3, 4, 5, 6])
         self.assertTrue(g.vs["label"] == ["1", "2", "3", "4", "5", "6"])
 
+        # Vertex names
+        edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
+        g = Graph.DataFrame(edges, use_vids=False)
+        self.assertTrue(g.vcount() == 6)
+
         # Vertex ids
         edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
         g = Graph.DataFrame(edges)
-        self.assertTrue(g.vcount() == 6)
-
-        edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
-        g = Graph.DataFrame(edges, use_vids=True)
         self.assertTrue(g.vcount() == 7)
 
         # Graph clone
@@ -404,7 +513,7 @@ class GeneratorTests(unittest.TestCase):
         g.es["w"] = [1.0] * g.ecount()
         df_edges = g.get_edge_dataframe()
         df_vertices = g.get_vertex_dataframe()
-        g_clone = Graph.DataFrame(df_edges, g.is_directed(), df_vertices, True)
+        g_clone = Graph.DataFrame(df_edges, g.is_directed(), df_vertices)
         self.assertTrue(df_edges.equals(g_clone.get_edge_dataframe()))
         self.assertTrue(df_vertices.equals(g_clone.get_vertex_dataframe()))
 
@@ -417,42 +526,45 @@ class GeneratorTests(unittest.TestCase):
             Graph.DataFrame(edges, vertices=pd.DataFrame())
         with self.assertRaisesRegex(TypeError, "integers"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]}).astype(str)
-            Graph.DataFrame(edges, use_vids=True)
+            Graph.DataFrame(edges)
         with self.assertRaisesRegex(ValueError, "negative"):
             edges = -pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
-            Graph.DataFrame(edges, use_vids=True)
+            Graph.DataFrame(edges)
         with self.assertRaisesRegex(TypeError, "integers"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3]}, index=["1", "2", "3"])
-            Graph.DataFrame(edges, vertices=vertices, use_vids=True)
+            Graph.DataFrame(edges, vertices=vertices)
         with self.assertRaisesRegex(ValueError, "negative"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3]}, index=[-1, 2, 3])
-            Graph.DataFrame(edges, vertices=vertices, use_vids=True)
+            Graph.DataFrame(edges, vertices=vertices)
         with self.assertRaisesRegex(ValueError, "sequence"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3]}, index=[1, 2, 4])
-            Graph.DataFrame(edges, vertices=vertices, use_vids=True)
+            Graph.DataFrame(edges, vertices=vertices)
         with self.assertRaisesRegex(TypeError, "integers"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
-            vertices = pd.DataFrame({0: [1, 2, 3]}, index=pd.MultiIndex.from_tuples([(1, 1), (2, 2), (3, 3)]))
-            Graph.DataFrame(edges, vertices=vertices, use_vids=True)
+            vertices = pd.DataFrame(
+                {0: [1, 2, 3]},
+                index=pd.MultiIndex.from_tuples([(1, 1), (2, 2), (3, 3)]),
+            )
+            Graph.DataFrame(edges, vertices=vertices)
         with self.assertRaisesRegex(ValueError, "unique"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 2]})
-            Graph.DataFrame(edges, vertices=vertices)
+            Graph.DataFrame(edges, vertices=vertices, use_vids=False)
         with self.assertRaisesRegex(ValueError, "already contains"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3], "name": [1, 2, 2]})
-            Graph.DataFrame(edges, vertices=vertices)
+            Graph.DataFrame(edges, vertices=vertices, use_vids=False)
         with self.assertRaisesRegex(ValueError, "missing from"):
             edges = pd.DataFrame({"source": [1, 2, 3], "target": [4, 5, 6]})
             vertices = pd.DataFrame({0: [1, 2, 3]}, index=[0, 1, 2])
-            Graph.DataFrame(edges, vertices=vertices, use_vids=True)
+            Graph.DataFrame(edges, vertices=vertices)
 
 
 def suite():
-    generator_suite = unittest.makeSuite(GeneratorTests)
+    generator_suite = unittest.defaultTestLoader.loadTestsFromTestCase(GeneratorTests)
     return unittest.TestSuite([generator_suite])
 
 
