@@ -1,6 +1,7 @@
 import random
 import unittest
-from igraph import *
+
+from igraph import Graph, set_random_number_generator
 
 
 class FakeRNG:
@@ -42,7 +43,7 @@ class RandomNumberGeneratorTests(unittest.TestCase):
 
 
 def suite():
-    random_suite = unittest.makeSuite(RandomNumberGeneratorTests)
+    random_suite = unittest.defaultTestLoader.loadTestsFromTestCase(RandomNumberGeneratorTests)
     return unittest.TestSuite([random_suite])
 
 
